/*
 * This file and everything in this package is covered under
 * the OPENdj Software License.  See the LICENSE file at the top
 * level of this package for full details.  A copy of the license
 * is also available at: http://opendj.org/docs/licenses/LICENSE.opendj
 *
 * Copyright 2001 DAX Interactive, LLC.
 *
 * $Author: jonathan $
 * $Revision: 1.2 $
 * $Header: /home/cvspublic/opendj/opendj-servercommon/src/org/opendj/publish/PublishSlot.java,v 1.2 2001/09/29 22:47:20 jonathan Exp $
 * $Log: PublishSlot.java,v $
 * Revision 1.2  2001/09/29 22:47:20  jonathan
 * Fixed DOS-style CRLFs that were all over the place.
 *
 * Revision 1.1.1.1  2001/09/29 20:38:49  jonathan
 * Imported opendj-servercommon
 *
 */
package org.opendj.publish;

import java.sql.Connection;
import java.sql.PreparedStatement;
import java.sql.ResultSet;
import java.sql.Date;
import java.sql.SQLException;
import java.util.List;
import java.util.Vector;
import org.opendj.util.Util;
import org.opendj.util.TimeUtil;
import org.opendj.util.TimePair;
import org.opendj.util.DBUtil;
import org.opendj.util.SchedBlock;
import org.opendj.util.debug;

public class PublishSlot {

    public int sid = -1;
    public int djid = -1;
    public String ipaddr = null;
    public String djemail = null;
    public String djname = null;
    public String showname = null;
    public int genre = -1;
    public String genreval = null;
    public int starttime = -1;
    public int day = -1;
    public int channelid = -1;
    public String channelpath = null;
    public String channelname = null;
    public boolean issingle = false;
    public Date singledate = null;
    public Date startlease = null;
    public Date endlease = null;

    public boolean shouldPrependToExisting = false;
    public boolean shouldAppendToExisting  = false;

    public PublishSlot() {}

    public void init ( ResultSet theRS ) throws SQLException {
        sid          = theRS.getInt("sid");
        djid         = theRS.getInt("djid");
        ipaddr       = theRS.getString("ipaddr");
        djemail      = theRS.getString("djemail");
        djname       = theRS.getString("djname");
        showname     = theRS.getString("showname");
        genre        = theRS.getInt("genre");
        genreval     = theRS.getString("genreval");
        starttime    = theRS.getInt("starttime");
        day          = theRS.getInt("day");
        channelid    = theRS.getInt("channelid");
        channelpath  = theRS.getString("channelpath");
        channelname  = theRS.getString("channelname");
        issingle     = (theRS.getInt("issingle") == 1);
        singledate   = theRS.getDate("singledate");
        startlease   = theRS.getDate("startlease");
        endlease     = theRS.getDate("endlease");
    }

    public boolean isNextPartOfSameShow ( PublishSlot p ) {
        // XXX MIDNIGHT ISSUE HERE
        return ( p != null &&
                 p.djid == this.djid &&
                 p.day  == this.day &&
                 p.channelid == this.channelid &&
                 p.issingle == this.issingle &&
                 ( ( p.issingle == true &&
                     p.singledate.equals(this.singledate) ) ||
                   ( p.issingle == false &&
                     p.startlease.equals(this.startlease) &&
                     p.endlease.equals(this.endlease) ) ) &&
                 (p.starttime + SchedBlock.SCHEDBLOCK_MINUTES) == this.starttime );
    }

    public void publish ( List theSIDList, Connection theConn ) throws SQLException {

        PreparedStatement anUpdatePS = null;
        String anUpdateSQL
            = "UPDATE Scheduled SET ispublished = 1 WHERE sid IN ( <sidlist> ) ";
        String aRealUpdateSQL = null;

        aRealUpdateSQL = Util.stringReplace(anUpdateSQL, 
                                            "<sidlist>",
                                            Util.listToString(theSIDList, ","));
        try {
            theConn.setAutoCommit(false);
            anUpdatePS = theConn.prepareStatement(aRealUpdateSQL);
            long aStart = -1;
            long anEnd = -1;
            int aNumMinutes = -1;

            if ( issingle ) {
                aStart = getSingleDateStartTime();
                aNumMinutes = SchedBlock.SCHEDBLOCK_MINUTES*theSIDList.size();
                anEnd = aStart + (Util.MILLIS_IN_MINUTE*aNumMinutes);

                // XXX Should be done in a transaction!  Need atomicity here!
                insertSlot(theConn, aStart, anEnd, 
                           new java.sql.Date(singledate.getTime()), starttime, (starttime+aNumMinutes), 0 );
                anUpdatePS.executeUpdate();

            } else {
                // We need to insert one published entity for each weekly show

                // lease ends at the end of the endlease date
                long anEndLeaseMillis = endlease.getTime() + Util.MILLIS_IN_DAY; 

                // Find the first show here...
                java.util.Date aBroadcastDate = Util.getNextDay( day, startlease );

                // XXX Should be done in a transaction!  Need atomicity here!
                int aNumWeeks = 1 + (int) Math.ceil((anEndLeaseMillis - aBroadcastDate.getTime()) / Util.MILLIS_IN_DAY / 7);
                while ( aBroadcastDate.getTime() < anEndLeaseMillis ) {
                    aBroadcastDate = Util.getTodayDate(aBroadcastDate);
                    aStart = aBroadcastDate.getTime() + (Util.MILLIS_IN_MINUTE*starttime);
                    aNumMinutes = SchedBlock.SCHEDBLOCK_MINUTES*theSIDList.size();
                    anEnd = aStart + (Util.MILLIS_IN_MINUTE*aNumMinutes);
                    insertSlot(theConn, aStart, anEnd, 
                               new java.sql.Date(aBroadcastDate.getTime()), 
                               starttime, (starttime+aNumMinutes),
                               aNumWeeks );
                    aBroadcastDate = Util.getNextDay( day, 
                                                      new Date(aBroadcastDate.getTime()
                                                               + (2*Util.MILLIS_IN_DAY) ) );
                }
                anUpdatePS.executeUpdate();
            }
            theConn.commit();

        } catch ( Exception e ) {
            debug.out("PublishSlot.publish: rolling back due to exception: " + e.toString());
            theConn.rollback();
            if ( e instanceof SQLException ) throw (SQLException) e;
            throw new SQLException("Unexpected non-SQLException: " + e.toString());

        } finally {
            DBUtil.closeStatement(anUpdatePS);
        }
    }

    private void setInsertPS ( PreparedStatement theInsertPS, 
                               long theStart, 
                               long theEnd,
                               int theNumWeeks ) throws SQLException {
        int i=1;
        theInsertPS.setInt   (i++, channelid);
        theInsertPS.setString(i++, channelpath);
        theInsertPS.setString(i++, channelname);
        theInsertPS.setInt   (i++, djid);
        theInsertPS.setInt   (i++, djid);
        theInsertPS.setString(i++, ipaddr);
        theInsertPS.setString(i++, djemail);
        theInsertPS.setString(i++, djname);
        theInsertPS.setString(i++, showname);
        theInsertPS.setInt   (i++, genre);
        theInsertPS.setString(i++, genreval);
        theInsertPS.setLong  (i++, theStart);
        theInsertPS.setLong  (i++, theEnd);
        theInsertPS.setInt   (i++, theNumWeeks);
    }

    private long getSingleDateStartTime() {
        long aTime = -1;
        if ( issingle ) {
            java.util.Date aSingleDate = new java.util.Date(singledate.getTime());
            aSingleDate = Util.getTodayDate(aSingleDate);
            aTime = aSingleDate.getTime() + (Util.MILLIS_IN_MINUTE*starttime);
            return aTime;
        }
        return -1;
    }

    protected class MiniSlot {
        public long start = -1;
        public long end = -1;
        public int edjid = -1;
        public int psid = -1;
        public MiniSlot () {}
        public void init ( ResultSet theRS ) throws SQLException {
            start = theRS.getLong("starttime");
            end = theRS.getLong("endtime");
            edjid = theRS.getInt("edjid");
            psid = theRS.getInt("psid");            
        }
    }

    private List getMatchingMiniSlots( Connection theConn,
                                       long theStart, 
                                       long theEnd, 
                                       int theDjid ) throws SQLException {
        List rList = new Vector();
        ResultSet aRS = null;
        PreparedStatement aPS = null;
        String aSQL
            = "SELECT psid, edjid, starttime, endtime "
            + "FROM PublishedSchedule "
            + "WHERE ( starttime = ? OR endtime = ? ) "
            + "AND djid = ? "
            + "ORDER BY starttime "
            ;
        MiniSlot m = null;
        
        try {
            aPS = theConn.prepareStatement(aSQL);
            aPS.setLong(1, theEnd);
            aPS.setLong(2, theStart);
            aPS.setInt (3, theDjid);
            aRS = aPS.executeQuery();
            while ( aRS.next() ) {
                m = new MiniSlot();
                m.init(aRS);
                rList.add(m);
            }
            return rList;
            
        } finally {
            DBUtil.closeRS(aRS);
            DBUtil.closeStatement(aPS);
        }
    }
    
    private void setStateFlags ( long theStart,
                                 long theEnd,
                                 List theMiniSlots ) {
        int stop=theMiniSlots.size();
        MiniSlot m = null;
        for ( int i=0; i<stop; i++ ) {
            m = (MiniSlot) theMiniSlots.get(i);
            if ( m.start == theEnd ) {
                shouldPrependToExisting = true;
            } else if ( m.end == theStart ) {
                shouldAppendToExisting = true;
            }
        }
    }
    
    private void insertSlot ( Connection theConn,
                              long theStart, 
                              long theEnd,
                              Date theDate,
                              int theStartMins,
                              int theEndMins,
                              int theNumWeeks ) throws SQLException {

        // Take care of DST issues here
        TimePair p = TimeUtil.DSTPolicy(theDate, theStartMins, theEndMins);
        theStart = p.start;
        theEnd = p.end;
        if ( p.isFatal() ) return;

        List someMatchingSlots = getMatchingMiniSlots(theConn, theStart, theEnd, djid);
        setStateFlags(theStart, theEnd, someMatchingSlots);
        int aNumMatches = someMatchingSlots.size();
        PreparedStatement aPS = null;
        PreparedStatement aDelPS = null;
        String aSQL = null;
        String aDelSQL = null;
        
        try {
            switch ( aNumMatches ) {
            case 0:
                // Just do a basic insert
                debug.out("PublishSlot: doing basic insert...");
                aSQL
                    = "INSERT INTO PublishedSchedule "
                    + "( channelid, channelpath, channelname, djid, edjid, ipaddr, djemail, "
                    + "djname, showname, genre, genreval, starttime, endtime, numweeks ) VALUES "
                    + "( ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ? ) "
                    ;
                aPS = theConn.prepareStatement(aSQL);
                setInsertPS(aPS, theStart, theEnd, theNumWeeks);
                aPS.executeUpdate();
                debug.out("PublishSlot: basic insert done.");
                return;
                
            case 1:
                // Was this a prepend or an append?
                if ( shouldPrependToExisting ) {
                    debug.out("PublishSlot: prepending...");
                    aSQL
                        = "UPDATE PublishedSchedule "
                        + "SET starttime = ?, numweeks = ? "
                        + "WHERE psid = ? "
                        ;
                    aPS = theConn.prepareStatement(aSQL);
                    aPS.setLong(1, theStart);
                    aPS.setInt (2, theNumWeeks);
                    aPS.setInt (3, ((MiniSlot) someMatchingSlots.get(0)).psid);
                    aPS.executeUpdate();
                    debug.out("PublishSlot: prepend done.");
                    return;
                    
                } else if ( shouldAppendToExisting ) {
                    debug.out("PublishSlot: appending...");
                    aSQL
                        = "UPDATE PublishedSchedule "
                        + "SET endtime = ?, numweeks = ? "
                        + "WHERE psid = ? "
                        ;
                    aPS = theConn.prepareStatement(aSQL);
                    aPS.setLong(1, theEnd);
                    aPS.setInt (2, theNumWeeks);
                    aPS.setInt (3, ((MiniSlot) someMatchingSlots.get(0)).psid);
                    aPS.executeUpdate();
                    debug.out("PublishSlot: append done.");
                    return;
                    
                } else {
                    // XXX bad...
                    debug.out("PublishSlot.insertSlot: only 1 mini slot found, but neither flag set!");
                    return;
                }
                
            case 2:
                debug.out("PublishSlot: merging gap...");
                aSQL
                    = "UPDATE PublishedSchedule "
                    + "SET endtime = ?, numweeks = ? "
                    + "WHERE psid = ? "
                    ;
                try {
                    theConn.setAutoCommit(false);
                    aPS = theConn.prepareStatement(aSQL);
                    aPS.setLong(1, ((MiniSlot) someMatchingSlots.get(1)).end);
                    aPS.setInt (2, theNumWeeks);
                    aPS.setInt (3, ((MiniSlot) someMatchingSlots.get(0)).psid);
                    
                    aDelSQL
                        = "DELETE FROM PublishedSchedule WHERE psid = ? ";
                    aDelPS = theConn.prepareStatement(aDelSQL);
                    aDelPS.setInt(1, ((MiniSlot) someMatchingSlots.get(1)).psid);
                    
                    // XXX Should be done in a transaction!  Need atomicity here!
                    aDelPS.executeUpdate();
                    aPS.executeUpdate();
                    theConn.commit();
                    debug.out("PublishSlot: merge done.");
                    return;

                } catch ( Exception e ) {
                    debug.out("PublishSlot.insertSlot: rolling back due to exception: " + e.toString());
                    theConn.rollback();
                    if ( e instanceof SQLException ) throw (SQLException) e;
                    throw new SQLException("Unexpected non-SQLException: " + e.toString());
                }
                
            default:
                // XXX bad...
                debug.out("PublishSlot.insertSlot: more than 2 mini slot found!");
                return;
            }
        } finally {
            DBUtil.closeStatement(aPS);
            DBUtil.closeStatement(aDelPS);
        }
    }
    
}
