/*
 * This file and everything in this package is covered under
 * the OPENdj Software License.  See the LICENSE file at the top
 * level of this package for full details.  A copy of the license
 * is also available at: http://opendj.org/docs/licenses/LICENSE.opendj
 *
 * Copyright 2001 DAX Interactive, LLC.
 *
 * $Author: jonathan $
 * $Revision: 1.1 $
 * $Header: /home/cvspublic/opendj/opendj-servercommon/src/org/opendj/util/AsyncProcess.java,v 1.1 2001/10/21 20:51:40 jonathan Exp $
 * $Log: AsyncProcess.java,v $
 * Revision 1.1  2001/10/21 20:51:40  jonathan
 * Added capability for processes to run asynchronously.
 *
 */
package org.opendj.util;

import java.io.InputStreamReader;
import java.io.BufferedReader;
import java.io.IOException;

/**
 * An internal class used by the ProcessUtil.execAsyncCommand
 * method.
 */
class AsyncProcess implements Runnable {

    private String itsCommand = null;
    private Thread itsNotifyThread = null;
    private volatile boolean isDone = false;
    private volatile Thread itsMainThread = null;
    private volatile Exception itsException = null;
    private volatile String itsOutput = null;
    private volatile Process itsProcess = null;

    /**
     * Create an AsyncProcess.
     *
     * @param theCommand The command that the AsyncProcess 
     * will execute.
     */
    public AsyncProcess ( String theCommand ) {
        itsCommand = theCommand;
        itsNotifyThread = null;
    }

    /**
     * Create an AsyncProcess.
     *
     * @param theCommand The command that the AsyncProcess 
     * will execute.
     * @param theNotifyThread The thread to interrupt when the command
     * has completed (whether it completed successfully or not).  
     * Typically this is the thread that created the AsyncProcess and
     * is waiting for it to finish.
     */
    public AsyncProcess ( String theCommand, Thread theNotifyThread ) {
        itsCommand = theCommand;
        itsNotifyThread = theNotifyThread;
    }

    /**
     * Execute the process.
     */
    public void run () {

        isDone = false;
        itsMainThread = Thread.currentThread();
        itsProcess = null;

        try {
            itsOutput = Util.EMPTY_STRING;
            BufferedReader aReader = null;
            String aLine = null;

            itsProcess = Runtime.getRuntime().exec(itsCommand);
            try {
                aReader = new BufferedReader(new InputStreamReader(itsProcess.getInputStream()));
                aLine = aReader.readLine();
                while ( aLine != null ) {
                    itsOutput += aLine + "\n";
                    aLine = aReader.readLine();
                }
                try {
                    itsProcess.waitFor();
                } catch ( InterruptedException e ) {}
                
            } finally { 
                try { aReader.close(); } catch ( Exception e2 ) {}
            }

        } catch ( Exception e ) {
            itsException = e;

        } finally {
            cleanup(false);
            if ( itsNotifyThread != null ) {
                itsNotifyThread.interrupt();
            }
        }
    }

    /**
     * Terminates the async process.
     */
    public void terminate () {
        if ( itsProcess != null ) {
            itsProcess.destroy();
        }
        cleanup(true);
    }

    private synchronized void cleanup (boolean isHardCleanup) {
        isDone = true;
        if ( isHardCleanup ) {
            if ( itsMainThread != null ) {
                itsMainThread.stop();
            }
        }
        itsMainThread = null;
    }

    public Exception getException () { return itsException; }

    public String getOutput () { return itsOutput; }

    public boolean getIsDone () { return isDone; }
}
