/*
 * This file and everything in this package is covered under
 * the OPENdj Software License.  See the LICENSE file at the top
 * level of this package for full details.  A copy of the license
 * is also available at: http://opendj.org/docs/licenses/LICENSE.opendj
 *
 * Copyright 2001 DAX Interactive, LLC.
 *
 * $Author: jonathan $
 * $Revision: 1.2 $
 * $Header: /home/cvspublic/opendj/opendj-servercommon/src/org/opendj/util/DBUtil.java,v 1.2 2001/09/29 22:47:20 jonathan Exp $
 * $Log: DBUtil.java,v $
 * Revision 1.2  2001/09/29 22:47:20  jonathan
 * Fixed DOS-style CRLFs that were all over the place.
 *
 * Revision 1.1.1.1  2001/09/29 20:38:48  jonathan
 * Imported opendj-servercommon
 *
 */
package org.opendj.util;

import java.io.FileInputStream;
import java.sql.DriverManager;
import java.sql.Connection;
import java.sql.ResultSet;
import java.sql.Statement;
import java.sql.PreparedStatement;
import java.sql.SQLException;
import java.util.Properties;

/**
 * Some utility functions to use along with the JDBC API.
 */
public class DBUtil {

    public static final String PROP_DB_DRIVER    = "djserver.jdbc.driver";
    public static final String PROP_DB_URL       = "djserver.jdbc.url";
    public static final String PROP_DB_USER      = "djserver.jdbc.user";
    public static final String PROP_DB_PASSWORD  = "djserver.jdbc.password";

    private static String DB_DRIVER    = null;
    private static String DB_URL       = null;
    private static String DB_USER      = null;
    private static String DB_PASSWORD  = null;

    private static String PROP_FILE = null;

    /**
     * Closes a Connection.  Any Exception that might have been generated 
     * is ignored.
     *
     * @param c the Connection to close.
     */
    public static void closeConnection ( Connection c ) {
        try { c.close(); } catch (Exception e) {}
    }

    /**
     * Closes a ResultSet.  Any Exception that might have been generated 
     * is ignored.
     *
     * @param rs the ResultSet to close.
     */
    public static void closeRS ( ResultSet rs ) {
        try { rs.close(); } catch (Exception e) {}
    }

    /**
     * Closes a Statement.  Any Exception that might have been generated 
     * is ignored.
     *
     * @param s the Statement to close.
     */
    public static void closeStatement ( Statement s ) {
        try { s.close(); } catch (Exception e) {}
    }

    /**
     * Formats a String for insertion into a database.
     *
     * @param s the String to format
     * @return the formatted String
     */
    public static String formatSQLString ( String s ) {
        if ( s == null ) {
            return "NULL";
        }
        return "'" + Util.stringReplace( s, "\'", "\\'" ) + "'";
    }

    public synchronized static Connection getConnection() throws SQLException {
        if ( DB_DRIVER == null ) {
            initDB();
        }
        return DriverManager.getConnection(DB_URL, DB_USER, DB_PASSWORD);
    }

    private synchronized static void initDB() throws SQLException {

        if ( DB_DRIVER != null ) return;

        try {
            Config aConfig = Config.global();
            DB_DRIVER   = aConfig.getProperty(Config.PROP_DB_DRIVER);
            DB_URL      = aConfig.getProperty(Config.PROP_DB_URL);
            DB_USER     = aConfig.getProperty(Config.PROP_DB_USER);
            DB_PASSWORD = aConfig.getProperty(Config.PROP_DB_PASSWORD);

        } catch ( Exception e ) {
            e.printStackTrace();
            throw new SQLException(e.toString());
        }

        try {
            Class.forName(DB_DRIVER).newInstance();

        } catch ( Exception e ) {
            e.printStackTrace();
            throw new SQLException(e.toString());
        }
    }

    public static boolean verifyUnique ( Connection theConn, 
                                         String theTable, 
                                         String theColName, 
                                         String theVal ) throws SQLException {

        if ( theVal == null ) return false;
        theVal = theVal.trim();
        
        PreparedStatement aPS = null;
        ResultSet aRS = null;
        String aSQL
            = "SELECT " + theColName
            + " FROM " + theTable
            + " WHERE LOWER(" + theColName + ") = LOWER( ? ) "
            ;

        try {
            aPS = theConn.prepareStatement(aSQL);
            aPS.setString(1, theVal);
            aRS = aPS.executeQuery();
            if ( aRS.next() ) return false;
            return true;
        } finally {
            DBUtil.closeStatement(aPS);
            DBUtil.closeRS(aRS);            
        }
    }

    public static boolean verifyUnique ( Connection theConn, 
                                         String theTable, 
                                         String theColName, 
                                         String theKeyColName,
                                         String theVal,
                                         int theKey ) throws SQLException {

        if ( theVal == null ) return false;
        theVal = theVal.trim();
        
        PreparedStatement aPS = null;
        ResultSet aRS = null;
        String aSQL
            = "SELECT " + theColName
            + " FROM " + theTable
            + " WHERE LOWER(" + theColName + ") = LOWER( ? ) "
            + " AND " + theKeyColName + " != ? "
            ;

        try {
            aPS = theConn.prepareStatement(aSQL);
            aPS.setString(1, theVal);
            aPS.setInt   (2, theKey);
            aRS = aPS.executeQuery();
            if ( aRS.next() ) return false;
            return true;
        } finally {
            DBUtil.closeStatement(aPS);
            DBUtil.closeRS(aRS);            
        }
    }

	public static Integer getInteger(
		ResultSet	theResultSet,
		int			theIndex)
			throws SQLException
	{
		int		value = theResultSet.getInt(theIndex);
		if (theResultSet.wasNull())
			return null;
		else
			return new Integer(value);
	}

	public static Long getLong(
		ResultSet	theResultSet,
		int			theIndex)
			throws SQLException
	{
		long	value = theResultSet.getLong(theIndex);
		if (theResultSet.wasNull())
			return null;
		else
			return new Long(value);
	}

	public static void setInt(
		PreparedStatement	theStmt,
		int					theCol,
		Integer				theValue)
			throws SQLException
	{
		if (null != theValue)
			theStmt.setInt(theCol, theValue.intValue());
		else
			theStmt.setNull(theCol, java.sql.Types.INTEGER);
	}

    /**
     * All methods in this class are static, so the constructor is private.
     * This class should never be instantiated.
     */
    private DBUtil () {}

}
