/*
 * This file and everything in this package is covered under
 * the OPENdj Software License.  See the LICENSE file at the top
 * level of this package for full details.  A copy of the license
 * is also available at: http://opendj.org/docs/licenses/LICENSE.opendj
 *
 * Copyright 2001 DAX Interactive, LLC.
 *
 * $Author: jonathan $
 * $Revision: 1.2 $
 * $Header: /home/cvspublic/opendj/opendj-servercommon/src/org/opendj/util/FormErrorBean.java,v 1.2 2001/09/29 22:47:20 jonathan Exp $
 * $Log: FormErrorBean.java,v $
 * Revision 1.2  2001/09/29 22:47:20  jonathan
 * Fixed DOS-style CRLFs that were all over the place.
 *
 * Revision 1.1.1.1  2001/09/29 20:38:48  jonathan
 * Imported opendj-servercommon
 *
 */
package org.opendj.util;

import java.util.List;
import java.util.Vector;
import java.util.Map;
import java.util.HashMap;
import java.util.Iterator;
import java.util.Date;
import javax.servlet.*;

public class FormErrorBean {

    public static final int ERR_REQUIRED        = 0;
    public static final int ERR_MATCH           = 1;
    public static final int ERR_SELECTION       = 2;
    public static final int ERR_SHORTPASSWORD   = 3;
    public static final int ERR_EASYPASSWORD    = 4;
    public static final int ERR_RADIO_SELECTION = 5;
    public static final int ERR_DATE            = 6;
    public static final int ERR_DATE_IN_PAST    = 7;
    public static final int ERR_DATE_TOO_EARLY  = 8;
    public static final int ERR_TOOLONG         = 9;
    public static final int ERR_ILLEGALCHAR     = 10;
    public static final int ERR_BOX_NOT_CHECKED = 11;
    public static final int ERR_NOT_UNIQUE      = 12;
    public static final int ERR_DATE_TOO_LATE   = 13;
    public static final int ERR_INVALID_EMAIL   = 14;
    public static final int ERR_TOOSHORT        = 15;
    public static final int ERR_INVALID_IP      = 16;
    public static final int ERR_NOT_UNIQUE_NR   = 17;

    private Map     itsFields       = null;
    private Map     itsErrors       = null;
    private List    itsAllErrors    = null;
    private String  itsGeneralError = null;
    private boolean itsHasErrors    = false;

    public FormErrorBean () {
        itsFields    = new HashMap();
        itsErrors    = new HashMap();
        itsAllErrors = new Vector();
        itsHasErrors = false;
    }

    public String getValue(String theFieldName) {
        return (String) itsFields.get(theFieldName);
    }

    public void setValue( String theFieldName, String theValue ) {
        itsFields.put(theFieldName, theValue);
    }

    public boolean getHasErrors() {
        return itsHasErrors;
    }

    public boolean validateChecked( ServletRequest request, String theFieldName, String theDisplayName ) {
        String aValue = request.getParameter(theFieldName);
        if ( aValue == null || aValue.length() == 0 ) {
            addError(ERR_BOX_NOT_CHECKED, theFieldName, theDisplayName);
            return false;
        }
        return true;
    }

    public boolean validateNonEmpty( ServletRequest request, 
                                     String theFieldName, 
                                     String theDisplayName, 
                                     int theMaxLength ) {
        return validateNonEmpty(request, theFieldName, theDisplayName, theMaxLength, -1);
    }

    public boolean validateNonEmpty(ServletRequest request,
                                    String theFieldName,
                                    String theDisplayName,
                                    int theMaxLength,
                                    int theMinLength) {
        String aValue = request.getParameter(theFieldName);

        if ( aValue == null || aValue.length() == 0 ) {
            addError(ERR_REQUIRED, theFieldName, theDisplayName);
            return false;

        } else if ( aValue.length() > theMaxLength ) {
            addError(ERR_TOOLONG, theFieldName, theDisplayName);
            return false;

        } else if ( aValue.indexOf("|") != -1 ) {
            addError(ERR_ILLEGALCHAR, theFieldName, theDisplayName);
            return false;

        } else if ( theMinLength != -1 && aValue.length() < theMinLength ) {
            addError(ERR_TOOSHORT, theFieldName, theDisplayName);
            return false;

        } else {
            setValue(theFieldName, aValue);
            return true;
        }
    }

    public boolean validatePassword ( ServletRequest request, 
                                      String theFieldName,
                                      String theDisplayName,
                                      String thePassword1,
                                      String thePassword2,
                                      int theMaxLength ) {

        String aVal1 = request.getParameter(thePassword1);
        String aVal2 = request.getParameter(thePassword2);
        if ( aVal1 == null || 
             aVal2 == null ||
             aVal1.length() == 0 || 
             aVal2.length() == 0 ) {
            addError(ERR_REQUIRED, theFieldName, theDisplayName);
            return false;
        }
        if ( !aVal1.equals(aVal2) ) {
            addError(ERR_MATCH, theFieldName, theDisplayName);
            return false;
        }
        if ( aVal1.length() < Util.MIN_PASSWORD_LENGTH ) {
            addError(ERR_SHORTPASSWORD, theFieldName, theDisplayName);
            return false;
        }
        if ( aVal1.length() - Util.countAlphanumericChars(aVal1) < 2 ) {
            addError(ERR_EASYPASSWORD, theFieldName, theDisplayName);
            return false;
        }
        if ( aVal1.length() > theMaxLength ) {
            addError(ERR_TOOLONG, theFieldName, theDisplayName);
        }
        return true;
    }

    public boolean validateSelectionMade ( ServletRequest request,
                                           String theFieldName,
                                           String theDisplayName) {
        String aSelectVal = request.getParameter(theFieldName);
        int aSelectIntVal = -1;
        try {
            aSelectIntVal = Integer.parseInt(aSelectVal);
        } catch ( Exception e ) {
            addError(ERR_SELECTION, theFieldName, theDisplayName);
            return false;
        }
        if ( aSelectIntVal == -1 ) {
            addError(ERR_SELECTION, theFieldName, theDisplayName);
            return false;
        }
        setValue(theFieldName, String.valueOf(aSelectIntVal));
        return true;
    }

    public boolean validateRadio ( ServletRequest request,
                                   String theFieldName, 
                                   String theDisplayName ) {

        String aRadioVal = request.getParameter(theFieldName);
        if ( aRadioVal == null || aRadioVal.length() == 0 ) {
            addError(ERR_RADIO_SELECTION, theFieldName, theDisplayName);
            return false;
        }
        setValue(theFieldName, aRadioVal);
        return true;
    }

    public boolean validateDate ( ServletRequest request,
                                  String theFieldName, 
                                  String theDisplayName ) {

        String aDateVal = request.getParameter(theFieldName);
        if ( aDateVal == null || aDateVal.length() == 0 ) {
            addError(ERR_DATE, theFieldName, theDisplayName);
            return false;
        }
        try { 
            DBFormat.ParseUIDate(aDateVal);
        } catch ( DBException dbe ) {
            addError(ERR_DATE, theFieldName, theDisplayName);
            return false;
        }
        setValue(theFieldName, aDateVal);
        return true;
    }

    public boolean validateSchedDate ( ServletRequest request,
                                       String theFieldName, 
                                       String theDisplayName ) {
        String aDateVal = request.getParameter(theFieldName);
        if ( aDateVal == null || aDateVal.length() == 0 ) {
            addError(ERR_DATE, theFieldName, theDisplayName);
            return false;
        }
        Date aSchedDate = null;
        try { 
            aSchedDate = DBFormat.ParseUIDate(aDateVal);
        } catch ( DBException dbe ) {
            addError(ERR_DATE, theFieldName, theDisplayName);
            return false;
        }

        // We now know it is a valid date -- but how many days 
        // in the future is it from us?
        long aNowTime = System.currentTimeMillis();
        long aSchedTime = aSchedDate.getTime();
        if ( aSchedTime < aNowTime ) {
            addError(ERR_DATE_IN_PAST, theFieldName, theDisplayName);
            return false;
        }
        if ( ((aSchedTime - aNowTime) / Util.MILLIS_IN_HOUR) < SchedBlock.ADVANCE_SCHED_HOURS ) {
            addError(ERR_DATE_TOO_EARLY, theFieldName, theDisplayName);
            return false;
        }
        if ( ((aSchedTime - aNowTime) / Util.MILLIS_IN_HOUR) > SchedBlock.MAX_ADVANCE_SCHED_HOURS ) {
            addError(ERR_DATE_TOO_LATE, theFieldName, theDisplayName);
            return false;
        }

        setValue(theFieldName, aDateVal);
        return true;
    }

    public void addError ( int theErrorType, String theFieldName, String theDisplayName ) {

        Integer anErrorTypeKey = new Integer(theErrorType);

        List someErrorsOfSameType = (List) itsErrors.get(anErrorTypeKey);

        if ( someErrorsOfSameType == null ) {
            someErrorsOfSameType = new Vector();
            itsErrors.put(anErrorTypeKey, someErrorsOfSameType);
        }

        someErrorsOfSameType.add(theDisplayName);
        itsHasErrors = true;
        itsAllErrors.add(theDisplayName);
    }

    public String hiliteError ( String theFieldName, String theDisplayName ) {
        if ( itsAllErrors.indexOf(theDisplayName) != -1 ) {
            return "<font color=\"#ff0000\">" 
                + theDisplayName 
                + "</font>"
                ;
        } else {
            return theDisplayName;
        }
    }

    public Iterator getErrorTypes() {
        return itsErrors.keySet().iterator();
    }

    public List getFieldsForErrorType( Integer theErrType ) {
        return (List) itsErrors.get(theErrType);
    }

    public void addGeneralError( String theMessage ) {
        itsGeneralError = theMessage;
        itsHasErrors = true;
    }

    public String getGeneralError() { return itsGeneralError; }

    public String gethtmlerror () {

        // Get all the different kinds of errors
        Iterator someErrorTypes = getErrorTypes();
        Integer  anErrorType    = null;
        Iterator someFields     = null;
        String   aFieldName     = null;
        String   rstr           = Util.EMPTY_STRING;

        try {
            // First check to see if we have a general error
            String aGeneralError = getGeneralError();
            if ( aGeneralError != null ) {
                rstr += "<br><h3>" + aGeneralError + "</h3>";
                return rstr;
            }

            while ( someErrorTypes.hasNext() ) {
                anErrorType = (Integer) someErrorTypes.next();
                switch ( anErrorType.intValue() ) {
                case FormErrorBean.ERR_REQUIRED:
                    rstr += "<br>The following fields are <b>required</b> "
                                + "but were left empty:";
                    break;
                    
                case FormErrorBean.ERR_INVALID_EMAIL:
                    rstr += "<br>The following fields are not valid email addresses:";
                    break;

                case FormErrorBean.ERR_INVALID_IP:
                    rstr += "<br>The following fields are not valid IP addresses:";
                    break;

                case FormErrorBean.ERR_TOOLONG:
                    rstr += "<br>The following fields are <b>too long</b>.  Please enter a shorter value.";
                    break;
                    
                case FormErrorBean.ERR_TOOSHORT:
                    rstr += "<br>The following fields are <b>too short</b>.  Please enter a longer value.";
                    break;

                case FormErrorBean.ERR_BOX_NOT_CHECKED:
                    rstr += "<br>You did not check some required checkboxes.";
                    break;
                    
                case FormErrorBean.ERR_ILLEGALCHAR:
                    rstr += "<br>The following fields contained <b>illegal characters</b>.";
                    break;

                case FormErrorBean.ERR_NOT_UNIQUE:
                    rstr += "<br>The following fields are <b>not unique</b> (someone else has already registered with those values).  Please pick a different value.";
                    break;

                case FormErrorBean.ERR_NOT_UNIQUE_NR:
                    rstr += "<br>The following fields were <b>not unique</b>.";
                    break;

                case FormErrorBean.ERR_MATCH:
                    rstr += "<br>The following fields did not match.  Please make sure"
                                + " that you type the same value in both fields.";
                    break;
                    
                case FormErrorBean.ERR_SELECTION:
                    rstr += "<br>No selection was made in the following drop-down boxes."
                                + " A selection is required.";
                    break;

                case FormErrorBean.ERR_RADIO_SELECTION:
                    rstr += "<br>No selection was made in the following radio buttons."
                                + " A selection is required.";
                    break;

                case FormErrorBean.ERR_DATE:
                    rstr += "<br>The following fields did not have a valid date.  Please use the MM/DD/YYYY format, for example 12/7/2000 for December 7th, 2000.";
                    break;

                case FormErrorBean.ERR_DATE_IN_PAST:
                    rstr += "<br>You cannot schedule a broadcast for a date in the past.  Please select another date.";
                    break;

                case FormErrorBean.ERR_DATE_TOO_EARLY:
                    rstr += "<br>You must schedule broadcasts at least " 
                                + SchedBlock.ADVANCE_SCHED_HOURS + " hours in advance.";
                    break;

                case FormErrorBean.ERR_DATE_TOO_LATE:
                    rstr += "<br>You cannot schedule a broadcast more than " 
                                + (SchedBlock.MAX_ADVANCE_SCHED_HOURS/24)
                                + " days in advance.";
                    break;

                case FormErrorBean.ERR_SHORTPASSWORD:
                    rstr += "<br>Your password must be at least "
                                + Util.MIN_PASSWORD_LENGTH
                                + " characters long.";
                    break;

                case FormErrorBean.ERR_EASYPASSWORD:
                    rstr += "<br>Your password must contain at least 2 "
                                + "non-alphanumeric characters.  This is a free "
                                + "broadcasting service, so security must be tight.  "
                                + "You wouldn't want anyone broadcasting and claiming "
                                + "to be you, would you?";
                    break;
                }
                
                someFields = getFieldsForErrorType(anErrorType).iterator();
                int i=0;
                while ( someFields.hasNext() ) {
                    if ( i++ > 0 ) rstr += ", ";
                    aFieldName = (String) someFields.next();
                    rstr += "<font color=\"#ff0000\">" + aFieldName +"</font>";
                }
            }
        } catch ( Exception e ) {
            System.err.println("FormErrorBean.gethtmlerror: " + e.toString());
            e.printStackTrace();
            return "An internal error occurred.  Please contact tech support.";
        }
        return rstr;
    }
}
