/*
 * This file and everything in this package is covered under
 * the OPENdj Software License.  See the LICENSE file at the top
 * level of this package for full details.  A copy of the license
 * is also available at: http://opendj.org/docs/licenses/LICENSE.opendj
 *
 * Copyright 2001 DAX Interactive, LLC.
 *
 * $Author: jonathan $
 * $Revision: 1.1.1.1 $
 * $Header: /home/cvspublic/opendj/opendj-servercommon/src/org/opendj/util/Karma.java,v 1.1.1.1 2001/09/29 20:38:48 jonathan Exp $
 * $Log: Karma.java,v $
 * Revision 1.1.1.1  2001/09/29 20:38:48  jonathan
 * Imported opendj-servercommon
 *
 */
package org.opendj.util;

import org.opendj.beans.onair.OnAirStatusBean;

public class Karma {

    public static final int PENALTY_NO_SHOW    = 50;
    public static final int BENEFIT_ONETIME    = 20;
    public static final int BENEFIT_WEEKLY     = 30;
    public static final int BENEFIT_WEEKLY_PER = 2;
    public static final int BENEFIT_RERUN      = 5;

    private int     itsKarmaDelta   = 0;
    private int     itsNumWeeks     = 0;
    private boolean itsWasLate      = false;
    private boolean itsLeftEarly    = false;
    private boolean itsNoShow       = false;
    private boolean itsWasSub       = false;
    private int     itsAvgListeners = 0;

    public Karma () {}

    public int getKarmaDelta () { return itsKarmaDelta; }
    public int getNumWeeks () { return itsNumWeeks; }
    public boolean getWasLate () { return itsWasLate; }
    public boolean getLeftEarly () { return itsLeftEarly; }
    public boolean getNoShow () { return itsNoShow; }
    public boolean getWasSub () { return itsWasSub; }
    public int getAvgListeners () { return itsAvgListeners; }

    public void init ( OnAirStatusBean theStatus, long theSampleMillis, boolean theWasRerun ) {
        
        long aPenalties = 0;
        long aBenefits = 0;

        itsNoShow = (theStatus.getOnairtime() <= 0);
        itsWasSub = theStatus.getWasSubstitute();
        long aNumBlocks = theStatus.getNumBlocks();
        float aNumHours = ((float) aNumBlocks * (float) SchedBlock.SCHEDBLOCK_MINUTES) / 60.0f;

        int aNoShowPenalty = (int) (aNumHours * (float) PENALTY_NO_SHOW);

        // Handle the rerun case
        if ( theWasRerun ) {
            itsWasLate = false;
            itsLeftEarly = false;
            itsNoShow = false;
            itsNumWeeks = 1;
            itsAvgListeners = (int) Math.ceil(theStatus.getAverageListeners());
            itsKarmaDelta = BENEFIT_RERUN + (getListenerBenefit(itsAvgListeners)/2);
            return;
        }

        // Handle the easy case
        if ( itsNoShow ) {
            itsWasLate = false; itsLeftEarly = false;
            aPenalties = aNoShowPenalty;
            aBenefits = 0;

        } else {
            // Gather data
            long aLateMillis = theStatus.getLateMillis();
            long anEarlyMillis = theStatus.getEarlyMillis();
            itsAvgListeners = (int) Math.ceil(theStatus.getAverageListeners());
            float anAverageDisconnectRate = theStatus.getAverageDisconnect(theSampleMillis);
            
            int aLatePenalty   = getTimePenalty(aLateMillis);
            itsWasLate = ( aLatePenalty > 0 );

            int anEarlyPenalty = getTimePenalty(anEarlyMillis);
            itsLeftEarly = ( anEarlyPenalty > 0 );

            int aDisconnectPenalty = getDisconnectPenalty(anAverageDisconnectRate);

            aPenalties = aLatePenalty + anEarlyPenalty + ((int) (aNumHours * (float) aDisconnectPenalty));

            itsNumWeeks = theStatus.getNumweeks();
            if ( itsNumWeeks > 0 && !itsWasSub ) {
                aBenefits = BENEFIT_WEEKLY + (BENEFIT_WEEKLY_PER * itsNumWeeks);
            } else {
                aBenefits = BENEFIT_ONETIME;
            }
            int aListenerBenefit = getListenerBenefit(itsAvgListeners);

            aBenefits += aListenerBenefit;
            aBenefits = (int) Math.ceil(((float) aBenefits) * aNumHours);
        }

        itsKarmaDelta = (int) (aBenefits - aPenalties);

        // Can never score worse than a no-show
        if ( itsKarmaDelta < ( -1 * aNoShowPenalty ) ) {
            itsKarmaDelta = ( -1 * aNoShowPenalty );
        }

        // Adjust for sub last
        if ( itsWasSub ) {
            itsKarmaDelta = (itsKarmaDelta >= 0) 
                ? (int) (1.15f * ((float)itsKarmaDelta))
                : (int) (-1.15f * ((float)itsKarmaDelta))
                ;
        }
    }

    private int getTimePenalty ( long theMillis ) {
        if ( theMillis < 15000 ) {
            return 0;
        } else if ( theMillis < 5 * Util.MILLIS_IN_MINUTE ) {
            return 5;
        } else if ( theMillis < 10 * Util.MILLIS_IN_MINUTE ) {
            return 10;
        } else {
            return 20;
        }
    } 

    private int getDisconnectPenalty ( float theRate ) {
        if ( theRate < 0.01 ) { // 36 seconds of down-time per hour
            return 0;
        } else if ( theRate < 0.025f ) {  // 90 seconds of down-time per hour
            return 10;
        } else if ( theRate < 0.05f ) {   // 180 seconds of down-time per hour
            return 20;
        } else {
            return 40;
        }
    }

    private int getListenerBenefit ( int theNumListeners ) {
        return ( 4 * theNumListeners );
    }
}
