/*
 * This file and everything in this package is covered under
 * the OPENdj Software License.  See the LICENSE file at the top
 * level of this package for full details.  A copy of the license
 * is also available at: http://opendj.org/docs/licenses/LICENSE.opendj
 *
 * Copyright 2001 DAX Interactive, LLC.
 *
 * $Author: jonathan $
 * $Revision: 1.3 $
 * $Header: /home/cvspublic/opendj/opendj-servercommon/src/org/opendj/util/ProcessUtil.java,v 1.3 2001/10/21 20:51:40 jonathan Exp $
 * $Log: ProcessUtil.java,v $
 * Revision 1.3  2001/10/21 20:51:40  jonathan
 * Added capability for processes to run asynchronously.
 *
 * Revision 1.2  2001/09/29 22:47:20  jonathan
 * Fixed DOS-style CRLFs that were all over the place.
 *
 * Revision 1.1.1.1  2001/09/29 20:38:48  jonathan
 * Imported opendj-servercommon
 *
 */
package org.opendj.util;

import java.io.IOException;
import java.io.InterruptedIOException;
import java.io.InputStreamReader;
import java.io.BufferedReader;

/**
 * A utility for creating and working with OS processes.
 */
public class ProcessUtil {

    /**
     * Simply executes the given command and waits for
     * it to exit.  No effort is made to read the output from
     * the command - if the process blocks because it's stdout
     * buffer cannot be flushed, this method may hang.
     *
     * @param theCommand The command to execute, including
     * any arguments.
     */
    public static int execWithWait( String theCommand ) 
        throws IOException, InterruptedIOException {

        Process aProcess = null;

        log("ProcessUtil.execWithWait('" + theCommand + "') called");
        aProcess = Runtime.getRuntime().exec(theCommand);
        
        try {
            System.err.print("ProcessUtil: exec'ed, waiting for exit...");
            aProcess.waitFor();
            log("exited OK");
        } catch ( InterruptedException e ) {}

        int anExitVal = aProcess.exitValue();
        log("ProcessUtil: returning " + anExitVal);
        return anExitVal;
    }

    /**
     * Executes a command, returns the output from the command as
     * a String.
     *
     * @param theCommand The command to execute, including any
     * arguments.
     * @return The output from the command.
     */
    public static String execStringCommand ( String theCommand ) 
        throws IOException, InterruptedIOException {
        
        Process aProcess = null;
        String rstr = Util.EMPTY_STRING;
        BufferedReader aReader = null;
        String aLine = null;

        log("ProcessUtil.execStringCommand('" + theCommand + "') called");

        aProcess = Runtime.getRuntime().exec(theCommand);
        try {
            aReader = new BufferedReader(new InputStreamReader(aProcess.getInputStream()));

            aLine = aReader.readLine();
            while ( aLine != null ) {
                rstr += aLine + "\n";
                aLine = aReader.readLine();
            }
            try {
                aProcess.waitFor();
            } catch ( InterruptedException e ) {}

        } finally { 
            try { aReader.close(); } catch ( Exception e2 ) {}
        }

        return rstr;
    }

    /**
     * Executes a command asynchronously, returns the output 
     * of the command as a String.
     *
     * @param theCommand The command to execute, including
     * any arguments.
     * @param theTimeLimit How long to wait (in milliseconds) 
     * for the command to complete.
     * @return The output from the command.
     * @exception AsyncProcessException If any IOException occurs
     * while running the command, or if the command times out.
     */
    public static String execAsyncCommand ( String theCommand, 
                                            long theTimeLimit ) 
        throws AsyncProcessException {

        log("ProcessUtil.execAsyncCommand('" + theCommand + "', " + theTimeLimit + ")");

        AsyncProcess ap = new AsyncProcess(theCommand, Thread.currentThread());
        Thread t = new Thread(ap);
        long aStartTime = System.currentTimeMillis();
        t.start();
        while ( !ap.getIsDone() ) {
            Util.sleep(250);
            if ( System.currentTimeMillis() - aStartTime > theTimeLimit ) {
                // Kill the process
                log("ProcessUtil.execAsyncCommand - timed out!");
                ap.terminate();
                throw new AsyncProcessException(AsyncProcessException.E_TIMEOUT, theTimeLimit);
            }
        }
        Exception e = ap.getException();
        if ( e != null ) {
            log("ProcessUtil.execAsyncCommand - exception occurred: " + e.toString());
            throw new AsyncProcessException(AsyncProcessException.E_EXCEPTION, e);
        }
        log("ProcessUtil.execAsyncCommand - returned OK");
        return ap.getOutput();
    }

    private static void log ( String s ) {
        System.err.println( (new java.util.Date()).toString() + ": " + s );
    }

    private ProcessUtil () {}
}
