/*
 * This file and everything in this package is covered under
 * the OPENdj Software License.  See the LICENSE file at the top
 * level of this package for full details.  A copy of the license
 * is also available at: http://opendj.org/docs/licenses/LICENSE.opendj
 *
 * Copyright 2001 DAX Interactive, LLC.
 *
 * $Author: jonathan $
 * $Revision: 1.2 $
 * $Header: /home/cvspublic/opendj/opendj-servercommon/src/org/opendj/util/SchedBlock.java,v 1.2 2001/09/29 22:47:20 jonathan Exp $
 * $Log: SchedBlock.java,v $
 * Revision 1.2  2001/09/29 22:47:20  jonathan
 * Fixed DOS-style CRLFs that were all over the place.
 *
 * Revision 1.1.1.1  2001/09/29 20:38:48  jonathan
 * Imported opendj-servercommon
 *
 */
package org.opendj.util;

import java.sql.Connection;
import java.sql.PreparedStatement;
import java.sql.ResultSet;
import java.sql.SQLException;
import java.util.Date;
import java.util.Calendar;
import org.opendj.share.DjBean;

public class SchedBlock {

    // These constants are used in the "status" column of the 
    // publishedschedule table
    public static final int STATUS_DJ_SCHEDULED       = 0;
    public static final int STATUS_NEEDSUB            = 1;
    public static final int STATUS_DJ_IS_SUB          = 2;
    public static final int STATUS_NEEDSUB_WITH_RERUN = 3;
    public static final int STATUS_DJ_SCHEDULED_RERUN = 4;
    public static final int STATUS_RS_SCHEDULED_RERUN = 5;

    public static final int ATTR_UNMODERATED    = (1<<0);
    public static final int ATTR_ONETIME        = (1<<1);
    public static final int ATTR_NONCONVERTIBLE = (1<<2);
    public static final int ATTR_SUBDJ          = (1<<3);

    public static final int SCHEDBLOCK_MINUTES = 15;

    public static final int ADVANCE_SCHED_HOURS = 48;
    public static final int MAX_ADVANCE_SCHED_HOURS = 24 * 180;

    public static final int MAX_WEEKLY_BLOCKS = 32;
    public static final int MAX_SINGLE_BLOCKS = 64;

    private static long SUBSTITUTE_REQUEST_LIMIT_MILLIS = Long.MAX_VALUE;
    private static long SUBSTITUTE_ACCEPT_LIMIT_MILLIS = Long.MAX_VALUE;
    private static long RERUN_SCHED_LIMIT_MILLIS = Long.MAX_VALUE;
    static {
        SUBSTITUTE_REQUEST_LIMIT_MILLIS
            = Util.MILLIS_IN_HOUR
            * Long.parseLong(Config.global().getProperty(Config.PROP_REQSUBLIMITHOURS));

        SUBSTITUTE_ACCEPT_LIMIT_MILLIS
            = Util.MILLIS_IN_HOUR
            * Long.parseLong(Config.global().getProperty(Config.PROP_MINSUBACCEPTHOURS));

        RERUN_SCHED_LIMIT_MILLIS
            = Util.MILLIS_IN_HOUR
            * Long.parseLong(Config.global().getProperty(Config.PROP_RERUNSCHEDLIMITHOURS));
    }

    public static final int WEEKLY_MINWEEKS = 4;
    public static final int WEEKLY_MAXWEEKS = 26;

    public static boolean validateStartTime ( int theStart, Date theDate ) {
        long aNowTime = System.currentTimeMillis();
        theDate = Util.getTodayDate(theDate);
        return ( theDate.getTime() + (theStart * Util.MILLIS_IN_MINUTE) > 
                 aNowTime + (ADVANCE_SCHED_HOURS * Util.MILLIS_IN_HOUR) );
    }

    public static Date getStartLeaseDate ( int theStart, int theDay ) {

        long aNowTime = System.currentTimeMillis();

        // Add ADVANCE_SCHED_HOURS to aNowTime -- this is when we'll start looking 
        // for the next opportunity to broadcast
        aNowTime += (ADVANCE_SCHED_HOURS * Util.MILLIS_IN_HOUR);
        
        // Get the day value of aNowTime
        Date aNowTimeDate = new Date(aNowTime);
        int aNowTimeDay = Util.getDay( aNowTimeDate );
        
        // Special case -- if aNowTimeDay is theDay, make sure that 
        // theStart is AFTER aNowTime
        if ( aNowTimeDay == theDay ) {
            if ( Util.getTodayDate( aNowTimeDate ).getTime() 
                 + (theStart * Util.MILLIS_IN_MINUTE) < aNowTime ) {
                // Shit, they just barely missed the 48 hour advance requirement
                // Return the date one week from today
                return new Date( aNowTime + (Util.MILLIS_IN_DAY*7) );
            } else {
                // They're OK to broadcast on this day -- return it.
                return aNowTimeDate;
            }

        } else {
            // Find the next occurence of theDay
            if ( theDay > aNowTimeDay ) {
                // theDay is further in this week... add the number of days
                return new Date( aNowTime + (Util.MILLIS_IN_DAY*(theDay-aNowTimeDay)) );
            } else {
                return new Date( aNowTime + (Util.MILLIS_IN_DAY*(theDay-aNowTimeDay+7)) );
            }
        }
    }

    public static Date getEndLeaseDate ( Date theStart, int theNumweeks ) {
        Calendar cal = Calendar.getInstance();
        cal.setTime(theStart);
        cal.add(Calendar.DATE, (7*(theNumweeks-1)));
        return cal.getTime();
    }

    public static String blocksToHours ( int theNumBlocks ) {
        String rstr = String.valueOf((float) theNumBlocks / 4.0f);
        if ( rstr.endsWith(".0") ) return rstr.substring(0, rstr.length()-2);
        return rstr;
    }

    public static boolean isWithinSubRequestTime ( long theShowTime ) {
        debug.out("Comparing st (" + theShowTime + ") - ct (" + System.currentTimeMillis() + ") = " + (theShowTime - System.currentTimeMillis()) + " > " + SUBSTITUTE_REQUEST_LIMIT_MILLIS);
        return ( theShowTime - System.currentTimeMillis() > SUBSTITUTE_REQUEST_LIMIT_MILLIS );
    }

    public static boolean isPastSubAcceptTime ( long theShowTime ) {
        return ( theShowTime < System.currentTimeMillis() + SUBSTITUTE_ACCEPT_LIMIT_MILLIS );
    }

    public static boolean isWithinRerunSchedTime ( long theShowTime ) {
        return ( theShowTime > System.currentTimeMillis() + RERUN_SCHED_LIMIT_MILLIS );
    }

    public static int validateWeekly ( Connection theConn,
                                       DjBean theDjBean,
                                       int theNumBlocks ) throws SQLException {
        PreparedStatement aPS = null;
        ResultSet aRS = null;
        String aSQL 
            = "SELECT COUNT(*) AS ct "
            + "FROM Scheduled "
            + "WHERE djid = ? "
            + "AND issingle = 0 "
            + "AND isactive = 1 "
            + "AND endlease > ? "
            ;
        long aNowTime = System.currentTimeMillis();
        java.sql.Date aNowDate = new java.sql.Date(aNowTime);
        int aBlockCount = -1;
        try {
            aPS = theConn.prepareStatement(aSQL);
            aPS.setInt (1, theDjBean.getdjid());
            aPS.setDate(2, aNowDate);
            aRS = aPS.executeQuery();
            if ( aRS.next() ) {
                aBlockCount = aRS.getInt(1);
                debug.out("bc=" + aBlockCount + ", mb=" + theDjBean.getmaxweeklyblocks());
                if ( (aBlockCount+theNumBlocks) >= theDjBean.getmaxweeklyblocks() ) {
                    return aBlockCount;
                }
            }
            return 0;

        } finally {
            DBUtil.closeStatement(aPS);
            DBUtil.closeRS(aRS);
        }
    }

    public static int validateSingle ( Connection theConn,
                                       DjBean theDjBean,
                                       int theNumBlocks ) throws SQLException {
        PreparedStatement aPS = null;
        ResultSet aRS = null;
        String aSQL 
            = "SELECT COUNT(*) AS ct "
            + "FROM Scheduled "
            + "WHERE djid = ? "
            + "AND issingle = 1 "
            + "AND isactive = 1 "
            ;
        long aNowTime = System.currentTimeMillis();
        int aBlockCount = -1;
        try {
            aPS = theConn.prepareStatement(aSQL);
            aPS.setInt (1, theDjBean.getdjid());
            aRS = aPS.executeQuery();
            if ( aRS.next() ) {
                aBlockCount = aRS.getInt(1);
                debug.out("bc=" + aBlockCount + ", mb=" + theDjBean.getmaxsingleblocks());
                if ( (aBlockCount+theNumBlocks) >= theDjBean.getmaxsingleblocks() ) {
                    return aBlockCount;
                }
            }
            return 0;

        } finally {
            DBUtil.closeStatement(aPS);
            DBUtil.closeRS(aRS);
        }
    }

    private SchedBlock() {}
}
