/*
 * This file and everything in this package is covered under
 * the OPENdj Software License.  See the LICENSE file at the top
 * level of this package for full details.  A copy of the license
 * is also available at: http://opendj.org/docs/licenses/LICENSE.opendj
 *
 * Copyright 2001 DAX Interactive, LLC.
 *
 * $Author: jonathan $
 * $Revision: 1.2 $
 * $Header: /home/cvspublic/opendj/opendj-servercommon/src/org/opendj/util/StringUtil.java,v 1.2 2001/09/29 22:47:20 jonathan Exp $
 * $Log: StringUtil.java,v $
 * Revision 1.2  2001/09/29 22:47:20  jonathan
 * Fixed DOS-style CRLFs that were all over the place.
 *
 * Revision 1.1.1.1  2001/09/29 20:38:48  jonathan
 * Imported opendj-servercommon
 *
 */
package org.opendj.util;

import java.io.*;
import java.util.*;
import javax.mail.internet.InternetAddress;
import org.apache.oro.text.regex.Perl5Compiler;
import org.apache.oro.text.regex.Perl5Matcher;
import org.apache.oro.text.regex.Pattern;
import org.opendj.util.log.*;

public class StringUtil
{
    private static final Perl5Compiler REGEX_COMPILER = new Perl5Compiler();
    private static final Perl5Matcher  REGEX_MATCHER  = new Perl5Matcher();

    public static final String EMAIL_REGEXP = "^[A-Za-z0-9_\\.\\-]+@([A-Za-z0-9_\\-]+\\.)+[A-Za-z]{2,3}$";
    public static final String EMAIL_DELIMITERS = ",; ";

    private static Pattern EMAIL_PATTERN = null;
    static {
        try {
            EMAIL_PATTERN = REGEX_COMPILER.compile(EMAIL_REGEXP);
        } catch ( Exception e ) {
            Logger.global().capture("StringUtil.static: could not create EMAIL_PATTERN", 
                                    e, 
                                    Logger.L_FIRE);
        }
    }

    /**
     * This function concatenates a directory or file to an existing directory
     * specification.  The concatenation is done properly regardless of whether
     * or not there is a trailing file separator on the first part or a leading
     * file seperator on the second part.
     * 
     * @param theFirstPart the directory to append to
     * @param theSecondPart the directory or file to append
     * @return the appended path
     */
    public static String concatFilePath(
        String    theFirstPart,
        String    theSecondPart)
    {
        return concatFilePath(theFirstPart, theSecondPart, File.separator);
    }
    
    /**
     * This function concatenates a directory or file to an existing directory
     * specification.  The concatenation is done properly regardless of whether
     * or not there is a trailing file separator on the first part or a leading
     * file seperator on the second part.  You may specify a different file
     * seperator than the OS specific file seperator.
     * 
     * @param theFirstPart the directory to append to
     * @param theSecondPart the directory or file to append
     * @param theFileSeperator the file seperator character to use
     * @return the appended path
     */
    public static String concatFilePath(
        String    theFirstPart,
        String    theSecondPart,
        String    theFileSeperator)
    {
        String    retVal = "";
        
        if (null != theFirstPart)
            retVal = theFirstPart;
        
        if (null != theSecondPart)
        {
            if (!theFirstPart.endsWith(theFileSeperator) && !theSecondPart.startsWith(theFileSeperator))
                retVal += theFileSeperator;
            retVal += theSecondPart;
        }
        
        return retVal;
    }


    public static Vector unpackList(String    list) {
        return unpackList(list, ",");
    }


    public static Vector unpackList(String    list, String theSeparator) {
        Vector rtnVal = new Vector();
        StringTokenizer    tokenizer = new StringTokenizer(list, theSeparator);
        
        while (tokenizer.hasMoreTokens())
        {
            rtnVal.addElement(tokenizer.nextToken());
        }
        return rtnVal;
    }

    public static String replaceString (String input, String pat, String rep)
    {
        String retVal = null;

        try 
        {
            
        if (input != null && input.length () > 0 && pat != null && pat.length () > 0 && rep != null)
        {            
            int idx, fromIndex, patLen = pat.length ();
            
            for (retVal = "", fromIndex = 0;
                 (idx = input.indexOf (pat, fromIndex)) != -1;
                 fromIndex = idx + patLen)
            {
                retVal += input.substring (fromIndex, idx);
                retVal += rep;
            }
            
            retVal += input.substring (fromIndex);
            
        }
        } catch (Exception e)
        {
            e.printStackTrace ();
            retVal = null;
        }
        
        return retVal;
    }

    public static boolean testRegExp( String input, String theRegExp ) {
        try {
            Pattern aPattern = REGEX_COMPILER.compile(theRegExp);
            return REGEX_MATCHER.matches(input, aPattern);
        } catch ( Exception e ) {
            Logger.global().capture("StringUtil.testRegExp: error compiling Regex",
                                    e,
                                    Logger.L_ERROR);
        }
        return false;
    }

    
    public static boolean testEmail( String input ) {
        return REGEX_MATCHER.matches(input, EMAIL_PATTERN);
    }

    public static boolean testIP ( String input ) {
        List someOctets = Util.stringToList(input, ".");
        if ( someOctets == null ) return false;
        if ( someOctets.size() != 4 ) return false;
        int[] someOctetInts = new int[4];
        for ( int i=0; i<4; i++ ) {
            try {
                someOctetInts[i] = Integer.parseInt((String) someOctets.get(i));
                debug.out("parsed octet: " + someOctetInts[i]);
            } catch ( Exception e ) {
                return false;
            }
            if ( someOctetInts[i] < 0 || someOctetInts[i] > 255 ) return false;
        }

        // Class C private
        if ( someOctetInts[0] == 192 && someOctetInts[1] == 168 ) {
            return false;
        }

        // Class B private
        if ( someOctetInts[0] == 172 && someOctetInts[1] >= 16 && someOctetInts[1] <= 31 ) {
            return false;
        }

        // Class A private
        if ( someOctetInts[0] == 10 ) return false;

        // localhost
        if ( someOctetInts[0] == 127 ) return false;

        // multicast and reserved addresses (Class D and E)
        if ( someOctetInts[0] >= 224 ) return false;

        // Other illegal addresses
        if ( someOctetInts[0] == 0 || someOctetInts[3] == 255 || someOctetInts[3] == 0 ) {
            return false;
        }

        return true;
    }
    
    public static String nz( String s ) {
        if ( s == null ) return "";
        return s;
    }

    public static String trunc( String s, int len ) {
        if ( s == null ) return null;
        if ( s.length() <= len ) return s;
        return s.substring(0,len);
    }

    // appends all the tokens to a string and returns it
    public static String tokenizerToString (StringTokenizer st, String pat)
    {
        String retVal = "";

        if (st != null && pat != null)
        {
            while (st.hasMoreTokens ())
                retVal += st.nextToken () + pat;
        }
        
        return retVal;
    }

    /**
     * Given a list of comma-separated email addresses, returns only the good ones
     */
    public static String getGoodEmailList( String theList, boolean theThrowException) {
        // Validate email addresses here?
        Vector aVerifiedToEmails = new Vector();
        // it's a comma-separated string
        StringTokenizer aStringTokenizer = new StringTokenizer(theList, EMAIL_DELIMITERS);
        while (aStringTokenizer.hasMoreTokens()) {
            String aEmail = (String)(aStringTokenizer.nextElement());
            boolean aIsMailOk = StringUtil.testEmail(aEmail);
           if (!aIsMailOk) {
                if (theThrowException) {
                    throw new IllegalArgumentException("Invalid email address");
                } else {
                    Logger.global().capture("Invalid email address : " + aEmail, null, Logger.L_INFO);
                }
            } else {
                aVerifiedToEmails.addElement(aEmail);
            }
        }
        
        String theGoodToEmail = Util.listToString(aVerifiedToEmails, ", ");
        return theGoodToEmail;
    }

    /**
     * Given an array of InternetAddress objects, returns only the good ones
     */
    public static String getGoodEmailList( InternetAddress[] theList, boolean theThrowException) {

        Vector aVerifiedToEmails = new Vector();

        for ( int i=0; i<theList.length; i++ ) {
            String aEmail = (String) theList[i].toString();
            boolean aIsMailOk = true; // StringUtil.testEmail(aEmail);
            if (!aIsMailOk) {
                if (theThrowException) {
                    throw new IllegalArgumentException("Invalid email address");
                } else {
                    Logger.global().capture("Invalid email address : " + aEmail, null, Logger.L_INFO);
                }
            }
            else {
                aVerifiedToEmails.addElement(aEmail);
            }
        }
        
        String theGoodToEmail = Util.listToString(aVerifiedToEmails, ", ");
        return theGoodToEmail;
    }

    /**
     * The 2 arguments are comma separated lists of emails; returns the difference between the first - second list
     */
    public static String stringListDiff(String someToEmails, String aEmailAlreadySent) {
        Vector aList1 = unpackList(someToEmails, EMAIL_DELIMITERS);
        Vector aList2 = unpackList(aEmailAlreadySent, EMAIL_DELIMITERS);
        Vector aResult = new Vector();
        for (int i = 0; i < aList1.size(); i++) {
            String aEmail = (String)(aList1.elementAt(i));
            if ( ! aList2.contains(aEmail) ) {
                aResult.addElement(aEmail);
            }
        }
        String aStringResult = Util.listToString(aResult, ", ");
        return aStringResult;
    }
}


