/*
 * This file and everything in this package is covered under
 * the OPENdj Software License.  See the LICENSE file at the top
 * level of this package for full details.  A copy of the license
 * is also available at: http://opendj.org/docs/licenses/LICENSE.opendj
 *
 * Copyright 2001 DAX Interactive, LLC.
 *
 * $Author: jonathan $
 * $Revision: 1.2 $
 * $Header: /home/cvspublic/opendj/opendj-servercommon/src/org/opendj/util/log/StatusLog.java,v 1.2 2001/09/29 22:47:20 jonathan Exp $
 * $Log: StatusLog.java,v $
 * Revision 1.2  2001/09/29 22:47:20  jonathan
 * Fixed DOS-style CRLFs that were all over the place.
 *
 * Revision 1.1.1.1  2001/09/29 20:38:49  jonathan
 * Imported opendj-servercommon
 *
 */
package org.opendj.util.log;

import java.io.*;
import java.text.*;
import java.util.*;

import org.opendj.util.Config;
import org.opendj.util.StringComparator;

public class StatusLog {

    
    private SimpleDateFormat    itsDateFormatter;
    private File                itsFile;
    private PrintWriter         itsWriter = null;
    private TreeMap             itsCheckpoints = null;
    private String              itsLastCheckpoint = null;
    private Date                itsLastCheckpointTimestamp = null;
    private boolean             itsCorrupted = false;
    
    private static final String CHECKPOINT = "CHECKPOINT";
    private static final String DATA = "DATA";
    private static final String SEP = "|";

    public static final String SUFFIX = ".status.log";

    public StatusLog(String  fileName) {
        Config aConfig = Config.global();
        itsFile = new File(aConfig.getProperty(Config.PROP_LOGDIR)
                           + File.separator
                           + fileName
                           + SUFFIX);       
        itsDateFormatter = new SimpleDateFormat ("yyyy.MM.dd hh:mm:ss.SSS");
    }
    
    //
    // status functions
    //
    public synchronized boolean exists() {
        return itsFile.exists();
    }
    
    //
    // operations
    //
    
    /**
     * Open the status log.
     * 
     * @param theAppendFlag if false, create a new file; otherwise append to existing file.
     */
    public synchronized void open(boolean theAppendFlag) 
        throws IOException, StatusLogException {
        close();
        if (theAppendFlag) getExistingContents();

        itsWriter = new PrintWriter
            (new FileWriter(itsFile.getAbsolutePath(), theAppendFlag));
    }
    
    public synchronized void checkpoint(String  theName)
        throws StatusLogException {
        output(CHECKPOINT, theName);
    }
    
    public synchronized void checkpoint(int  theCheckpoint)
        throws StatusLogException {
        output(CHECKPOINT, String.valueOf(theCheckpoint));
    }
    
    public synchronized void checkpoint(long  theCheckpoint)
        throws StatusLogException {
        output(CHECKPOINT, String.valueOf(theCheckpoint));
    }

    public synchronized void data(String  theData)
        throws StatusLogException {
        output(DATA, theData);
    }
    
    public synchronized void output(String  theOperation,
                                    String  theData)
        throws StatusLogException {
        if (null == itsWriter) {
            throw new StatusLogException("The status log is not open");
        }
        
        itsWriter.println(theOperation + SEP + itsDateFormatter.format(new Date()) + SEP + theData);
        itsWriter.flush();
    }
    
    private void getExistingContents()
        throws StatusLogException, IOException {

        itsCheckpoints = new TreeMap(new StringComparator());
        
        if (!exists()) return; // nothing to get!

        // inital checkpoint
        StatusLogCheckpoint currentCheckpoint = new StatusLogCheckpoint();
        itsCheckpoints.put("", currentCheckpoint);
        itsLastCheckpoint = "";
        itsLastCheckpointTimestamp = null;
        
        BufferedReader  reader = null;
        // read file
        try {
            reader = new BufferedReader(new FileReader(itsFile));
            
            for (;;) {
                String  operation = null;
                Date    timestamp = null;
                String  data = null;
                
                String line = reader.readLine();
                if ( line == null ) break;
                
                line = line.trim();
                if (line.length() == 0) continue;
                
                // get operation
                int index = line.indexOf(SEP);
                if ( index != -1 ) {
                    operation = line.substring(0, index);
                    line = line.substring(index + 1);
                }
                
                // get timestamp & data
                index = line.indexOf(SEP);
                if ( index != -1 ) {
                    try {
                        timestamp = itsDateFormatter.parse(line.substring(0, index));
                        data = line.substring(index + 1);
                    } catch (ParseException e) {
                        // line is corrupted; will be handled below
                    }
                }
                
                // 
                // create data structures
                //
                if ( operation == null ) {
                    // Current checkpoint is corrupted; can't continue
                    currentCheckpoint.setCorrupted();
                    setCorrupted();
                    break;
                }
                
                if (operation.equals(CHECKPOINT)) {
                    if ( data == null || data.length() == 0 ) {
                        // can't start next checkpoint
                        currentCheckpoint.setCorrupted();
                        setCorrupted();
                    }
                    
                    currentCheckpoint = new StatusLogCheckpoint();
                    currentCheckpoint.setTimeStamp(timestamp);
                    itsCheckpoints.put(data, currentCheckpoint);
                    itsLastCheckpoint = data;
                    itsLastCheckpointTimestamp = timestamp;

                } else {
                    StatusLogEntry entry = new StatusLogEntry();
                    entry.setOperation(operation);
                    if ( timestamp == null ) {
                        entry.setCorrupted();
                        setCorrupted();

                    } else {
                        entry.setTimeStamp(timestamp);
                        entry.setData(data);
                        currentCheckpoint.addEntry(entry);
                    }
                }
            }    

        } catch (FileNotFoundException e) {
            throw new StatusLogException("Internal logic error: staus log file not found in StatusLog.getExistingContents()");

        } finally {
            try { if (reader != null) reader.close(); } catch (Exception e) {}
        }
    }
    
    public void close() {
        try { if ( itsWriter != null ) itsWriter.close(); } catch ( Exception e ) {}
        itsWriter = null;
        itsCheckpoints = null;
        itsLastCheckpoint = null;
        itsLastCheckpointTimestamp = null;
        itsCorrupted = false;
    }
    
    public String getLastCheckpoint() {
        return itsLastCheckpoint;
    }
    
    public Date getLastCheckpointTimestamp() {
        return itsLastCheckpointTimestamp;
    }

    public int getLastCheckpointAsInt() {
        if (itsLastCheckpoint == null || itsLastCheckpoint.equals(""))
            return 0;
        else
            return Integer.valueOf(itsLastCheckpoint).intValue();
    }
    
    public long getLastCheckpointAsLong() {
        if (itsLastCheckpoint == null || itsLastCheckpoint.equals(""))
            return 0;
        else
            return Long.valueOf(itsLastCheckpoint).longValue();
    }

    public void delete() {
        close();
        if (!itsFile.delete())
            Logger.global().capture("StatusLog.delete: " + itsFile, null, Logger.L_FIRE);
    }
    
    private void setCorrupted() {
        itsCorrupted = true;
    }
    
    public boolean isCorrupted() {
        return itsCorrupted;
    }
    
    public String getFileName() {
        return itsFile.getAbsolutePath();
    }
    
    public StatusLogCheckpoint getCheckpoint(String theCheckpointID) {
        return (StatusLogCheckpoint)itsCheckpoints.get(theCheckpointID.toString());
    }
    
    public StatusLogCheckpoint getCheckpoint(int theCheckpointID) {
        if (theCheckpointID == 0)
            return getCheckpoint("");
        else
            return getCheckpoint(String.valueOf(theCheckpointID));
    }
    
    public Vector getCheckpointEntries(String theCheckpointID) {
        StatusLogCheckpoint ckpoint = getCheckpoint(theCheckpointID);
        if (ckpoint != null)
            return ckpoint.getEntries();
        else
            return null;
    }

    public Vector getCheckpointEntries(int theCheckpointID) {
        if (theCheckpointID == 0)                
            return getCheckpointEntries("");
        else
            return getCheckpointEntries(String.valueOf(theCheckpointID));
    }

    public Iterator getCheckpointIterator() {
        return itsCheckpoints.keySet().iterator();
    }
}
