/*
 * This file and everything in this package is covered under
 * the OPENdj Software License.  See the LICENSE file at the top
 * level of this package for full details.  A copy of the license
 * is also available at: http://opendj.org/docs/licenses/LICENSE.opendj
 *
 * Copyright 2001 DAX Interactive, LLC.
 *
 * $Author: jonathan $
 * $Revision: 1.1.1.1 $
 * $Header: /home/cvspublic/opendj/opendj-servercommon/src/org/opendj/util/mail/AsyncMailClient.java,v 1.1.1.1 2001/09/29 20:38:49 jonathan Exp $
 * $Log: AsyncMailClient.java,v $
 * Revision 1.1.1.1  2001/09/29 20:38:49  jonathan
 * Imported opendj-servercommon
 *
 */
package org.opendj.util.mail;

import java.util.TreeSet;
import javax.mail.internet.*;

import java.sql.*;
import java.util.*;
import java.io.*;

import org.opendj.util.log.*;
import org.opendj.util.*;
import org.opendj.util.service.*;

public class AsyncMailClient implements Service {

    private static Logger LOG = null;

    private static final int    JOB_SIZE = 10;
    private static final int    SLEEP_INTERVAL = 30000;
    private static final long   ITERATION_LOG_INTERVAL = 250;
    private static final int    MAX_SEND_ATTEMPTS = 8;

    private StatusLog           itsStatusLog = null;	
	
    private static final int    CKPOINT_INITIAL = 0;
    private static final String OP_MAIL_SENT = "MailSent";
    private static final String OP_START_MAIL = "AttemptMail";
    private final static String STATUS_LOG_NAME = "AsyncMailClient";

    private long itsIterationCount = 0;
    private volatile boolean isDone = false;
    private volatile Thread itsMainThread = null;

    public static void main(String[] args) {
		LOG = new Logger("AsyncMailClient");
        try {
            AsyncMailClient client = new AsyncMailClient();
            client.run();
        } catch (Exception e) {
            LOG.capture("AsyncMailClient.main: Abnormal termination", e, Logger.L_FIRE);
        }
	}

    public void initService () throws ServiceException {
        LOG = new Logger("AsyncMailClient");
    }

    public void stopService () {
        isDone = true;
        LOG.capture("AsyncMailClient shutting down...", null, Logger.L_INFO);
        if ( itsMainThread != null ) {
            try { itsMainThread.interrupt(); } catch ( Exception e ) {}
            try { itsMainThread.join(3500); } catch ( Exception e ) {}
            if ( itsMainThread.isAlive() ) {
                LOG.capture("main thread did not exit gracefully, terminating with Thread.stop", 
                            null, Logger.L_WARNING);
                itsMainThread.stop();
                LOG.capture("halted due to Thread.stop", null, Logger.L_WARNING);
            }
        }
    }
    
    public AsyncMailClient() {
        itsStatusLog = new StatusLog(STATUS_LOG_NAME);
    }

	public void run() {

        isDone = false;
        itsMainThread = Thread.currentThread();
        LOG.capture("AsyncMailClient.main: starting on " + (new java.util.Date()).toString(), 
                    null, 
                    Logger.L_INFO);

        itsIterationCount = 0;
        try {
            while ( !isDone ) {
                if ( ((++itsIterationCount) % ITERATION_LOG_INTERVAL) == 0 ) {
                    LOG.capture("AsyncMailClient.run: performing iteration #" + itsIterationCount,
                                null,
                                Logger.L_INFO);
                }
                doIteration();
                Util.sleep(SLEEP_INTERVAL);
            }
        } catch ( Exception e ) {
            LOG.capture("AsyncMailClient.run: unexpected exception", e, Logger.L_ERROR);
        }
        LOG.capture("AsyncMailClient halted.", null, Logger.L_INFO);
	}
	
    /**
     * This function obtains a block of async mail ids to process
     */
    private Vector getIDList() throws SQLException {
        Connection          conn = null;
        PreparedStatement   ps = null;
        ResultSet           rs = null;
        Vector              jobIDs = new Vector();

        try {
            conn = getConnection();
            conn.setAutoCommit(false);
            conn.setTransactionIsolation(Connection.TRANSACTION_SERIALIZABLE);

            String sql
                = "SELECT amid "
                + "FROM AsyncMail "
                + "WHERE status = ? "
                + "AND errorcount <= ? "
                + "AND nextsend < ? "
                + "ORDER BY queuetime";
            ps = conn.prepareStatement(sql);
            ps.setInt (1, MailManager.STATUS_UNSENT);
            ps.setInt (2, MAX_SEND_ATTEMPTS);
            ps.setLong(3, System.currentTimeMillis());
            rs = ps.executeQuery();
            
            for (int i = 0; i < JOB_SIZE && rs.next(); i++) {
                jobIDs.addElement(new Long(rs.getLong(1)));
            }

            if (0 == jobIDs.size()) return null;
            
            for (int i = 0; i < jobIDs.size(); i++) {
                long    id = ((Long)jobIDs.elementAt(i)).longValue();
                setStatus(conn, id, MailManager.STATUS_SENDING, Util.getLocalHostAddr());
            }
            conn.commit();
        
        } catch ( Exception e ) {
            conn.rollback();
            LOG.capture("AsyncMailClient.getIDList", e, Logger.L_ERROR);
            if ( e instanceof SQLException ) throw (SQLException) e;
            throw new SQLException("AsyncMailClient.getIDList: unexpected non-SQLException:"
                                   + e.toString());

        } finally {
            DBUtil.closeRS(rs);
            DBUtil.closeStatement(ps);
            DBUtil.closeConnection(conn);
        }
        
        return jobIDs;
    }
                   
    private void doIteration()
        throws SQLException, IOException, StatusLogException, AddressException, MailException {
        /*
            First, complete any existing operation that may have been interrupted
            while in process.
            Second, check the database for any new jobs and process them.
        */
        Vector      mailIDs;
        TreeSet     currentJobStatus = new TreeSet(new StringComparator());
        
        mailIDs = recover(currentJobStatus);
        if (mailIDs != null) {
            processJob(mailIDs, currentJobStatus);
        }
        
        for (;;) {
            mailIDs = getIDList();
            if (null != mailIDs && mailIDs.size() > 0) {
                processJob(mailIDs, null);
            } else {
                break;
            }
        }
    }

    /**
     * Produce a string that describes a mail id and destination address
     * 
     * @param theID the mail ID
     * @param theAddress the address that the mail is sent to (cannot be null)
     * @return a string to place in the status log
     */
    private String getStatusTag( long    theID, 
                                 String  theAddress ) {
        return new Long(theID).toString() + ":" + theAddress.trim();
    }

            
    private void processJob( Vector      theIDs,                 // Vector<Long>
                             TreeSet     theCurrentJobStatus )   // completed emails
        throws IOException, SQLException, StatusLogException, AddressException, MailException {
        /*
        This function can be called in one of two states:
            - we are recovering from a previous job (status file exists && 
              theCurrentJobStatus is non-null)
            - we are starting a new job
                
        If we starting a new job, we first create a new status file.  For each 
        id, obtain the data from the database, and prepare the email message.
        For each "to" address, log that a delivery attempt is being started, 
        deliver the message, and than log that delivery is complete.
        
        When all messages have been succesfully delivered, update the status of
        the mail item in the database, and delete the log file.
        */

        // consistency check
        if (null != theCurrentJobStatus && !itsStatusLog.exists())
            throw new MailException("Consistency check failure in AsyncMailClient: status log does not exist!");
        
        if (!itsStatusLog.exists()) itsStatusLog.open(false);
        
        Connection          aConn = null;
        PreparedStatement   ps = null;
        ResultSet           rs = null;
        String fromEmail = null;
        String subject = null;
        String toEmails = null;
        String message = null;
        int errorcount = 0;
        String aSQL
            = "SELECT message, toemails, subject, fromemail, errorcount "
            + "FROM AsyncMail "
            + "WHERE amid=? and status=? and client=?";
        try {
            aConn = getConnection();
            ps = aConn.prepareStatement(aSQL);
        
            for (int i = 0; i < theIDs.size(); i++) {        
                
                long id = ((Long)theIDs.elementAt(i)).longValue();
            
                try {
                    //
                    // get mail info
                    //
                    ps.setLong  (1, id);
                    ps.setInt   (2, MailManager.STATUS_SENDING);
                    ps.setString(3, Util.getLocalHostAddr());
                    rs = ps.executeQuery();

                    if ( !rs.next() ) {
                        // mail item has been taken care of somehow...
                        continue;
                    }

                    int j=1;
                    message    = rs.getString(j++);
                    toEmails   = rs.getString(j++);
                    subject    = rs.getString(j++);
                    fromEmail  = rs.getString(j++);
                    errorcount = rs.getInt(j++);

                } catch ( Exception e ) {
                    LOG.capture("AsyncMailClient.processJob", e, Logger.L_FIRE);
                    continue;

                } finally {
                    DBUtil.closeRS(rs);
                    DBUtil.closeStatement(ps);
                }
                
                try {
                    sendEmails(id, toEmails, fromEmail, subject, message, theCurrentJobStatus);
                } catch ( Exception e ) {
                    if ( e instanceof AddressException ) {
                        // There's no hope for this email ... a bad to/from email address
                        setFatal(aConn, id);
                    } else {
                        // Increment error count for this email
                        incrementError(aConn, id, errorcount);
                    }
                    
                    // This MUST be at errorlevel L_MAILER or else we could create an 
                    // INFINITE cascade of emails to send, because the LOG.capture
                    // would otherwise queue up another email, which could in turn fail.
                    LOG.capture("AsyncMailClient.processJob: "
                                + "error calling sendEmails, skipping asyncid #" + id, 
                                e, 
                                LOG.L_MAILER);
                    continue;
                }
                setStatus(aConn, id, MailManager.STATUS_SENT, MailManager.UNASSIGNED_CLIENT);
            } // for all mail ids

        } finally {
            DBUtil.closeRS(rs);
            DBUtil.closeStatement(ps);
            DBUtil.closeConnection(aConn);
        }

        itsStatusLog.delete();        
    }

    /**
     * Check for any unprocessed work to do.  This function will leave things in one 
     * two states:
     *   - recovery required: the return value is non-null, the current job status 
     *     contains info on any emails that were succesfully sent, and the status log
     *     is open and ready for logging succesful emails OR
     *   - no recovery required: the return value is null, and there is no status log
     *     in existence.
     * 
     * @param theCurrentJobStatus a TreeSet which will have email status tags inserted into it
     * @return a vector of longs if recovery is required, or null if recopvery is not required
     */
    private Vector recover(TreeSet theCurrentJobStatus) 
        throws SQLException, IOException, StatusLogException {

        Vector              retVal = null;
        
        //
        // obtain any unprocessed mail ids from db
        //
        Connection          conn = null;
        PreparedStatement   ps = null;
		ResultSet           rs = null;
        String sql
            = "SELECT amid "
            + "FROM AsyncMail "
            + "WHERE status=? and client=? "
            + "and errorcount <= ? "
            + "and nextsend < ? ";
        
        try {
            conn = getConnection();
            ps = conn.prepareStatement(sql);
            ps.setInt   (1, MailManager.STATUS_SENDING);
            ps.setString(2, Util.getLocalHostAddr());
            ps.setInt   (3, MAX_SEND_ATTEMPTS);
            ps.setLong  (4, System.currentTimeMillis());
            rs = ps.executeQuery();
            
            if (rs.next()) {
                retVal = new Vector();
                do {
                    retVal.addElement(DBUtil.getLong(rs, 1));
                } while (rs.next());
            }

        } catch ( Exception e ) {
            LOG.capture("AsyncMailClient.recover", e, Logger.L_FIRE);
            if ( e instanceof SQLException ) throw (SQLException) e;
            throw new SQLException("AsyncMailClient.recover: unexpected non-SQLException: "
                                   + e.toString());

        } finally  {
            DBUtil.closeRS(rs);
            DBUtil.closeStatement(ps);
            DBUtil.closeConnection(conn);
        }
        
        //
        // if there are no current results, delete log file and return
        //
        if (null == retVal) {
            if (itsStatusLog.exists())
                itsStatusLog.delete();
            return null;
        }
        
        //
        // check for any pending results
        //
        itsStatusLog.open(true);
        StatusLogCheckpoint ckpoint = itsStatusLog.getCheckpoint(CKPOINT_INITIAL);

        if ( ckpoint != null ) {
            Vector  entries = ckpoint.getEntries();

            for (int i = 0; i < entries.size(); i++) {
                StatusLogEntry  entry = (StatusLogEntry)entries.elementAt(i);
                if (entry.getOperation().equals(OP_MAIL_SENT))
                    theCurrentJobStatus.add(entry.getData());
            }
        }
        
        return retVal;
    }

    private void sendEmails( long    theID, 
                             String  theToEmails, 
                             String  theFromEmail, 
                             String  theSubject, 
                             String  theMessage,
                             TreeSet theCurrentJobStatus)
        throws AddressException, StatusLogException, MailException {

		// sanity check if there is at least 1 destination!
		if (theToEmails == null || theToEmails.length() == 0) return;
		
        StringTokenizer st = new StringTokenizer(theToEmails, ",");
        MailPiece aPiece = new MailPiece();
        aPiece.subject   = theSubject;
        aPiece.message   = theMessage;
        aPiece.fromEmail = new InternetAddress(theFromEmail);
        aPiece.toEmails = new InternetAddress[1];
        String aMailErr = null;
        			
        int anErrCount = 0;
        while (st.hasMoreTokens()) {
            String  toAddress = st.nextToken();
            String  statusTag = getStatusTag(theID, toAddress);
                
            if (null != theCurrentJobStatus && theCurrentJobStatus.contains(statusTag))
                continue;   // already sent this e-mail...
                    
            aPiece.toEmails[0] = new InternetAddress(toAddress);

            // log that you are attempting to send e-mail
		    itsStatusLog.output(OP_START_MAIL, statusTag);

            MailManager.global().send(aPiece);

            // mail succesfully sent (as far as we know); log it as done
            itsStatusLog.output(OP_MAIL_SENT, statusTag);
		}
    }
	
    private void setStatus( Connection  theConn, 
                            long        theID, 
                            int         theStatus,
                            String      theClient )
        throws SQLException {

        PreparedStatement   ps = null;
        try {
            String  sql = "UPDATE AsyncMail SET status=?, client=? WHERE amid=?";
            ps = theConn.prepareStatement(sql);
            ps.setInt   (1, theStatus);
            ps.setString(2, theClient);
            ps.setLong  (3, theID);
            ps.executeUpdate();
        } finally {
            DBUtil.closeStatement(ps);
        }
	}

    private void setFatal ( Connection theConn, long theID ) throws SQLException {
        PreparedStatement ps = null;
        String sql
            = "UPDATE AsyncMail "
            + "SET status=? "
            + "WHERE amid=?";
        try {
            ps = theConn.prepareStatement(sql);
            ps.setInt (1, MailManager.STATUS_ERROR);
            ps.setLong(2, theID);
            ps.executeUpdate();
        } finally {
            DBUtil.closeStatement(ps);
        }
    }

    private void incrementError ( Connection theConn, 
                                  long theID, 
                                  int theErrorCount ) throws SQLException {
        PreparedStatement ps = null;
        String sql
            = "UPDATE AsyncMail "
            + "SET errorcount=?, nextsend=? "
            + "WHERE amid=?";
        long aNextSendTime
            = System.currentTimeMillis() 
               + (Util.MILLIS_IN_HOUR*(1L<<((long)theErrorCount)));
        theErrorCount++;
        try {
            ps = theConn.prepareStatement(sql);
            ps.setInt (1, theErrorCount);
            ps.setLong(2, aNextSendTime);
            ps.setLong(3, theID);
            ps.executeUpdate();
        } finally {
            DBUtil.closeStatement(ps);
        }
    }

    private Connection getConnection() throws SQLException {
        return DBUtil.getConnection();
    }
}
