/*
 * This file and everything in this package is covered under
 * the OPENdj Software License.  See the LICENSE file at the top
 * level of this package for full details.  A copy of the license
 * is also available at: http://opendj.org/docs/licenses/LICENSE.opendj
 *
 * Copyright 2001 DAX Interactive, LLC.
 *
 * $Author: jonathan $
 * $Revision: 1.2 $
 * $Header: /home/cvspublic/opendj/opendj-servercommon/src/org/opendj/util/mail/MailManager.java,v 1.2 2001/09/29 22:47:20 jonathan Exp $
 * $Log: MailManager.java,v $
 * Revision 1.2  2001/09/29 22:47:20  jonathan
 * Fixed DOS-style CRLFs that were all over the place.
 *
 * Revision 1.1.1.1  2001/09/29 20:38:49  jonathan
 * Imported opendj-servercommon
 *
 */
package org.opendj.util.mail;

import java.sql.*;
import org.opendj.util.*;
import org.opendj.util.log.*;

/*
 * Based on a sendmail class by:
 * Godmar Back
 * University of Utah, Computer Systems Lab, January 1996
 *
 * a simple applet that sends you mail when your page is accessed
 */

import java.io.*;
import java.net.InetAddress;
import java.util.Properties;
import java.util.Date;

import javax.mail.*;
import javax.mail.internet.*;

public class MailManager {

    String      itsLocalHost;
    boolean     addrealhostname = true;
    String      itsLastLine = "";

    public final static int STATUS_UNSENT  = 0;
    public final static int STATUS_SENDING = 1;
    public final static int STATUS_SENT    = 2;
    public final static int STATUS_ERROR   = 3;

    public static final int Q_THROW     = 0;
    public static final int Q_NO_THROW  = 1;

    private static String MAILMANAGER_SMTPHOST = null;
    public  static final String HEADER_MAILER_NAME  = "X-Mailer";
    private static String HEADER_MAILER_VALUE = "OpenDJ_Site_Mail_Service";

    public static final String UNASSIGNED_CLIENT = "unassigned";

    private static MailManager itsMailManager = null;

    public static MailManager global() {
        if ( itsMailManager == null ) {
            itsMailManager = new MailManager();
        }
        return itsMailManager;
    }

    private MailManager() {
        if ( MAILMANAGER_SMTPHOST == null ) {
            MAILMANAGER_SMTPHOST = Config.global().getProperty(Config.PROP_SMTPHOST);
        }
    }

    public void collect(BufferedReader in, Message msg)
					throws MessagingException, IOException {
        String line;
        StringBuffer sb = new StringBuffer();
        while ((line = in.readLine()) != null) {
            sb.append(line);
            sb.append("\n");
        }
        
        // If the desired charset is known, you can use
        // setText(text, charset)
        msg.setText(sb.toString());
    }

    public void deliver ( InternetAddress theFromEmail, 
                          InternetAddress[] theToEmails, 
                          String theSubject, 
                          String theMessage, 
                          int theThrowException ) throws MailException {
        MailPiece aMail = new MailPiece();
        aMail.fromEmail = theFromEmail;
        aMail.toEmails = theToEmails;
        aMail.subject = theSubject;
        aMail.message = theMessage;
        deliver(aMail);
    }

    public void deliver ( MailPiece theMail ) throws MailException {
        // For now this is "send" we can change it to "queue" later if we want
        // but I think the AsyncMailClient needs more fault-tolerance.

        // OK, I've added some considerable FT to the AsyncMailClient...
        queue(theMail);
    }

    public synchronized void send( MailPiece theMail ) throws MailException {

        // Uncomment this ONLY if there are network connectivity problems
        // and ONLY for debugging purposes
        /*
        if ( true ) {
            Logger.global().capture("MailManager.send: " + theMail, null, Logger.L_DEBUG);
            return;
        }
        */

        Properties aMailSessionProps = null;
        Session aMailSession = null;
        Message aMessage = null;
        BufferedReader aBodyReader = null;

        try {
            aMailSessionProps = System.getProperties();
            aMailSessionProps.put("mail.smtp.host", theMail.host);

            aMailSession = Session.getDefaultInstance(aMailSessionProps, null);
            aMessage = new MimeMessage(aMailSession);
            
            aMessage.setFrom(theMail.fromEmail);
            if ( theMail.toEmails != null ) {
                aMessage.setRecipients( Message.RecipientType.TO, 
                                        theMail.toEmails);
            }
            if ( theMail.ccEmails != null ) {
                aMessage.setRecipients( Message.RecipientType.CC, 
                                        theMail.ccEmails);
            }
            if ( theMail.bccEmails != null ) {
                aMessage.setRecipients( Message.RecipientType.BCC, 
                                        theMail.bccEmails);
            }
            aMessage.setSubject(theMail.subject);
            aMessage.setHeader(HEADER_MAILER_NAME, HEADER_MAILER_VALUE);
            aMessage.setSentDate(new Date());

            aBodyReader = new BufferedReader
                (new StringReader(theMail.message));

            collect(aBodyReader, aMessage);

            Logger.global().capture("Sending mail:\n'"  + theMail.message + "'\n", 
                                    null, 
                                    Logger.L_MAILER);
            Transport.send(aMessage);

        } catch ( Exception e ) {
            if ( theMail == null ) {
                Logger.global().capture("MailManager.send(NULL-MAIL)",
                                        e,
                                        Logger.L_MAILER);
            } else {
                Logger.global().capture("MailManager.send("
                                        + theMail + ")",
                                        e,
                                        Logger.L_MAILER);
            }
            throw new MailException("Could not send email", e);
        }
    }

    public void queue ( MailPiece theMailPiece ) throws MailException {
        queue(theMailPiece.fromEmail, 
              theMailPiece.toEmails, 
              theMailPiece.subject, 
              theMailPiece.message,
              MailManager.Q_NO_THROW);
    }

	/**
	 * Stores the email data in the database queue, marking it "UNSENT".
	 * If the flag theThrowException is true, throws an exception if any of the destination
	 * emails is invalid; if false, simply discard the wrong email and send the message to
	 * all other destinations.
	 */	
	public void queue ( InternetAddress theFromEmail, 
                        InternetAddress[] theToEmails, 
                        String theSubject, 
                        String theMessage, 
                        int theThrowException ) throws MailException {
        
		String theGoodToEmails
            = StringUtil.getGoodEmailList( theToEmails, 
                                           (theThrowException == Q_THROW) );

        Connection aConn = null;
        PreparedStatement ps = null;
        String aSQL = "INSERT INTO AsyncMail "
            + "( fromemail, subject, toemails, message, queuetime, "
            + " status, errorcount, nextsend, client ) "
            + "VALUES ( ?, ?, ?, ?, ?, " + STATUS_UNSENT  + ", 0, ?, ? )";

        try {
			// aConn = DBManager.global().getConnection();
            aConn = DBUtil.getConnection();
			
            ps = aConn.prepareStatement(aSQL);
            ps.setString(1, theFromEmail.toString());
            ps.setString(2, theSubject);
            ps.setString(3, theGoodToEmails);
            ps.setString(4, theMessage);
            ps.setLong  (5, System.currentTimeMillis());
            ps.setLong  (6, System.currentTimeMillis());
            ps.setString(7, MailManager.UNASSIGNED_CLIENT);
            ps.executeUpdate();
        } catch (Exception e) {
            Logger.global().capture("MailManager.queue",
                                    e,
                                    Logger.L_MAILER);
        } finally {
            DBUtil.closeConnection(aConn);
			DBUtil.closeStatement(ps);
		}
	}

    public String getSmtpHost () {
        return MAILMANAGER_SMTPHOST;
    }
}
