/*
 * This file and everything in this package is covered under
 * the OPENdj Software License.  See the LICENSE file at the top
 * level of this package for full details.  A copy of the license
 * is also available at: http://opendj.org/docs/licenses/LICENSE.opendj
 *
 * Copyright 2001 DAX Interactive, LLC.
 *
 * $Author: jonathan $
 * $Revision: 1.1.1.1 $
 * $Header: /home/cvspublic/opendj/opendj-servercommon/src/org/opendj/util/service/ServiceManager.java,v 1.1.1.1 2001/09/29 20:38:49 jonathan Exp $
 * $Log: ServiceManager.java,v $
 * Revision 1.1.1.1  2001/09/29 20:38:49  jonathan
 * Imported opendj-servercommon
 *
 */
package org.opendj.util.service;

import java.io.PrintWriter;
import java.io.IOException;
import java.util.List;
import java.util.Vector;
import java.util.Iterator;
import org.opendj.util.Util;
import org.opendj.util.Config;
import org.opendj.util.debug;
import org.opendj.util.log.Logger;

public class ServiceManager implements Runnable {

    public static final String SERVICEMANAGER_LOGNAME = "ServiceManager";
    public static final long SLEEP_INTERVAL = 5000;
    public static final long JOIN_MILLIS = 5000;
    public static final int MAX_RESTART_ATTEMPTS = 5;

    private static Logger LOG = null;

    private List itsServices = null;
    private volatile Thread itsMainThread = null;
    private volatile boolean isDone = false;
    private ServiceListener itsListener = null;

    public static void main ( String[] args ) {
        LOG = new Logger(SERVICEMANAGER_LOGNAME);
        ServiceManager sm = new ServiceManager();
        try {
            sm.go();
        } catch ( Exception e ) {
            LOG.capture("Exception in ServiceManager.go", e, Logger.L_FIRE);
        }        
    }

    public ServiceManager() {}

    public void go () {

        isDone = false;
        itsMainThread = Thread.currentThread();

        Config aConfig = Config.global();
        int aListenerPort = Integer.parseInt(aConfig.getProperty(Config.PROP_SERVICEPORT));
        String anAuthPassword = aConfig.getProperty(Config.PROP_SERVICEAUTHPASSWORD);

        Runtime.getRuntime().addShutdownHook(new Thread(this));

        List someClassNames = getClassNames();
        int aNumServices = someClassNames.size();
        itsServices = new Vector();
        int i;
        Iterator iter = null;
        ServiceStruct aServiceStruct = null;

        // Start the service listener
        itsListener = new ServiceListener(this, aListenerPort, anAuthPassword);
        itsListener.go();

        // Create and initialize the services
        for ( i=0; i<aNumServices; i++ ) {
            initService(((String) someClassNames.get(i)).trim());
        }

        // For services that were successfully created and initialized,
        // start them
        iter = itsServices.iterator();
        aServiceStruct = null;
        while ( iter.hasNext() ) {
            aServiceStruct = (ServiceStruct) iter.next();
            LOG.capture("ServiceManager: starting " + aServiceStruct.service.getClass().getName(),
                        null,
                        Logger.L_INFO);
            aServiceStruct.thread.start();
        }

        boolean skipSleep = true;
        while ( !isDone ) {
            // Monitor services every 5 seconds
            if ( !skipSleep ) {
                Util.sleep(SLEEP_INTERVAL);
                skipSleep = false;
            } else {
                Util.sleep(250); // "yield" a bit in case we get caught in a tight loop
            }
            if ( isDone ) break;

            synchronized (itsServices) {
                iter = itsServices.iterator();
                while ( iter.hasNext() ) {
                    
                    aServiceStruct = (ServiceStruct) iter.next();
                    
                    if ( !aServiceStruct.thread.isAlive() ) {
                        itsServices.remove(aServiceStruct);
                        if ( aServiceStruct.restartCount > MAX_RESTART_ATTEMPTS ) {
                            LOG.capture("Service died maximum (" + MAX_RESTART_ATTEMPTS + ") "
                                        + "times, NOT restarting: " 
                                        + aServiceStruct.service.getClass().getName(), 
                                        null,
                                        Logger.L_ERROR);
                        } else {
                            LOG.capture("Service died, restarting: " 
                                        + aServiceStruct.service.getClass().getName(), 
                                        null,
                                        Logger.L_ERROR);                    
                            aServiceStruct = initService(aServiceStruct.service.getClass().getName());
                            if ( aServiceStruct != null ) {
                                aServiceStruct.restartCount++;
                                aServiceStruct.thread.start();
                            }
                        }
                        skipSleep = true;
                        break;
                    }
                }
            }
        }
        cleanup();
    }

    public void run () {
        // This method simply implements the shutdown hook
        isDone = true;
        if ( itsMainThread != null && itsMainThread.isAlive() ) {
            itsMainThread.interrupt();
        }
    }

    private List getClassNames() {
        Config aConfig = Config.global();
        String someClassNameStrings = aConfig.getProperty(Config.PROP_SERVICECLASSES);
        return Util.stringToList(someClassNameStrings, ",");
    }

    public ServiceStruct initService ( String theClassName ) {

        // Make sure no other service exists by this name
        if ( findService(theClassName) != null ) {
            LOG.capture("Service already running: " + theClassName, null, Logger.L_WARNING);
            return null;
        }

        Service aService = null;
        try {
            aService = (Service) Class.forName(theClassName).newInstance();
        } catch ( Exception e ) {
            LOG.capture("Could not create service: " + theClassName, e, Logger.L_FIRE);
            return null;
        }

        try {
            aService.initService();
        } catch ( Exception e ) {
            LOG.capture("initService failed for service: " + theClassName, e, Logger.L_FIRE);
            return null;
        }

        Thread aServiceThread = new Thread(aService);
        ServiceStruct aServiceStruct = new ServiceStruct(aService, aServiceThread);
        synchronized (itsServices) {
            itsServices.add(aServiceStruct);
        }
        return aServiceStruct;
    }

    public boolean stopService ( String theClassName ) {

        if ( theClassName == null || theClassName.length() == 0 ) return false;

        if ( theClassName.equals("ALL") ) {
            cleanup();
            Util.sleep(15000);
            Runtime.getRuntime().halt(1); // should never return
            return true;
        }

        ServiceStruct aServiceStruct = null;

        synchronized (itsServices) {
            aServiceStruct = findService(theClassName);
            if ( aServiceStruct != null ) {
                stopService(aServiceStruct);
                return true;
            }
        }
        return false;
    }

    private ServiceStruct findService ( String theClassName ) {
        int aNumServices = itsServices.size();
        ServiceStruct aServiceStruct = null;
        for ( int i=0; i<aNumServices; i++ ) {
            aServiceStruct = (ServiceStruct) itsServices.get(i);
            if ( aServiceStruct == null ) continue;
            debug.out("ServiceManager: Comparing ss='" + aServiceStruct.service.getClass().getName() 
                      + "' to '" + theClassName + "'");
            if ( aServiceStruct.service.getClass().getName().equals(theClassName) ) {
                debug.out("ServiceManager: Found it!");
                return aServiceStruct;
            }
        }
        return null;
    }

    private void stopService ( ServiceStruct theServiceStruct ) {

        LOG.capture("ServiceManager: stopping " + theServiceStruct.service.getClass().getName(),
                    null,
                    Logger.L_INFO);

        itsServices.remove(theServiceStruct);
        theServiceStruct.service.stopService();

        // Give the service 5 seconds to die
        if ( theServiceStruct.thread.isAlive() ) {
            try {
                theServiceStruct.thread.join(JOIN_MILLIS);
            } catch ( InterruptedException ie ) {}
                
            // Forcibly kill it
            LOG.capture("ServiceManager: " + theServiceStruct.service.getClass().getName() 
                        + " did not end gracefully (within " + JOIN_MILLIS + "), forcibly "
                        + " terminating with Thread.stop",
                        null,
                        Logger.L_ERROR);
            theServiceStruct.thread.stop();
        }
    }
    
    private void cleanup () {

        itsListener.finish();

        // We're done, give all the thread a chance to clean up...
        ServiceStruct aServiceStruct = null;
        synchronized (itsServices) {
            int aNumServices = itsServices.size();
            for ( int i=0; i<aNumServices; i++ ) {
                aServiceStruct = (ServiceStruct) itsServices.get(0);

                // This should remove the service, so we can continue
                // to use index 0 to stop the next service
                stopService(aServiceStruct); 
            }
        }
    }

    public void printRunningServices ( PrintWriter theWriter ) throws IOException {
        ServiceStruct aServiceStruct = null;
        synchronized (itsServices) {
            int aNumServices = itsServices.size();
            for ( int i=0; i<aNumServices; i++ ) {
                aServiceStruct = (ServiceStruct) itsServices.get(i);
                theWriter.println(aServiceStruct.service.getClass().getName() 
                                  + " (restarted " + aServiceStruct.restartCount + " times)");
            }
        }
    }

    public class ServiceStruct {
        public Service service  = null;
        public Thread  thread   = null;
        public int restartCount = 0;
        public ServiceStruct(Service service,
                             Thread thread) {
            this.service = service;
            this.thread = thread;
            this.restartCount = 0;
        }
        public boolean equals(Object o) {
            if ( o instanceof ServiceStruct ) {
                ServiceStruct s = (ServiceStruct) o;
                return ( s != null && 
                         s.service != null && 
                         service != null &&
                         s.service.getClass().getName().equals(service.getClass().getName()) );
            }
            return false;
        }
    }

    public void log ( String s, Exception theException, int theLogLevel ) {
        LOG.capture(s, theException, theLogLevel);
    }
}
