/*
 * This file and everything in this package is covered under
 * the OPENdj Software License.  See the LICENSE file at the top
 * level of this package for full details.  A copy of the license
 * is also available at: http://opendj.org/docs/licenses/LICENSE.opendj
 *
 * Copyright 2001 DAX Interactive, LLC.
 *
 * $Author: jonathan $
 * $Revision: 1.2 $
 * $Header: /home/cvspublic/opendj/opendj-servercommon/src/org/opendj/util/template/TemplateParserReader.java,v 1.2 2001/09/29 22:47:20 jonathan Exp $
 * $Log: TemplateParserReader.java,v $
 * Revision 1.2  2001/09/29 22:47:20  jonathan
 * Fixed DOS-style CRLFs that were all over the place.
 *
 * Revision 1.1.1.1  2001/09/29 20:38:49  jonathan
 * Imported opendj-servercommon
 *
 */
package org.opendj.util.template;

import java.io.*;
import java.util.*;

import org.opendj.util.*;
import org.opendj.util.log.*;

public class TemplateParserReader {

    public static final String SYMBOL_NOT_FOUND = "<symbol-not-found>";
    public static final String SYMBOL_DELIM     = "$";

    private String          itsBuffer  = null;
    private BufferedReader  itsReader  = null;
    private Map             itsSymbols = null;

    public TemplateParserReader ( String  theFileName,
                                  Map     theSymbols ) 
        throws IOException {

        File aFile = new File(theFileName);
        itsReader = new BufferedReader(new FileReader(aFile));
        itsSymbols = theSymbols;
    }

    public TemplateParserReader ( String theFileName, 
                                  String theSymbolsFile, 
                                  Map theAdditionalSymbols ) 
        throws IOException, TemplateParserException {
        
        // Parse symbol file
        if ( theSymbolsFile != null ) {
            itsSymbols = parseSymbols(theSymbolsFile);

            // Add additional symbols.
            if ( theAdditionalSymbols != null ) {
                itsSymbols.putAll(theAdditionalSymbols);
            }

        } else {
            itsSymbols = theAdditionalSymbols;
        }

        File aFile = new File(theFileName);
        itsReader = new BufferedReader(new FileReader(aFile));
    }

    /**
     * This function reads the entire file and parses it into a String.
     * Due to the obvious memory inefficiency of this function, its usage
     * should be avoided wherever possible.
     * 
     * This function closes the reader when done.
     * 
     * @return the parsed file
     */
    public String getString() throws IOException {
        try {
            StringBuffer    retValue = new StringBuffer();
            String          line = null;
            
            while (null != (line = readLine())) {
                retValue.append(line);
                retValue.append("\n");
            }
            
            return retValue.toString();
        } finally {
            close();
        }
    }
    
    public String readLine() throws IOException {
        return translateLine(itsSymbols, itsReader.readLine());
    }

    public void close() throws IOException {
        itsReader.close();
    }

	private static Map parseSymbols ( String theFileName )
        throws IOException, TemplateParserException {

        TreeMap rtnVal = new TreeMap(new StringComparator());
        
        BufferedReader  reader = null;
        int             lineNumber = 0;
        
//        try
//        {            
            reader = new BufferedReader(new FileReader(theFileName));
//        }
//        catch (Exception e)
//        {
//            System.out.println("Error opening symbol file " + theFileName + ": " + e);
//            if ( e instanceof IOException ) throw (IOException) e;
//            throw new IOException(e.toString());
//        }

        try {
            for (;;) {
                lineNumber++;
                String  line = reader.readLine();
                
                if (null == line) return rtnVal;
                
                StringTokenizer tokenizer = new StringTokenizer(line, "=", true);
                
                String  name = tokenizer.nextToken();
                String  operator = tokenizer.nextToken();
                String  value = tokenizer.nextToken();
                
                if (!operator.equals("="))
                    throw new TemplateParserException("Illegal symbol file format on line " + lineNumber);
                
                rtnVal.put(name.trim(), value.trim());
            }
            
//        } catch (Exception e) {
//            System.out.println("Error in substitution file " + theFileName + " on line " + lineNumber);
//            if ( e instanceof IOException ) throw (IOException) e;
//            throw new IOException(e.toString());
        
        } finally {
            try { reader.close(); } catch (Exception e) {}
        }
    }

    private static String translateLine(Map theSymbolMap, String theLine) {

        if ( theSymbolMap == null || theLine == null ) return theLine;

        StringTokenizer st = new StringTokenizer(theLine, SYMBOL_DELIM, true);
        String rstr = "";

        while (st.hasMoreTokens()) {
            String token = st.nextToken();
    
            if (token.equals(SYMBOL_DELIM)) {
                token = st.nextToken();
                // Double SYMBOL_DELIM means a literal SYMBOL_DELIM
                if (token.equals(SYMBOL_DELIM)) {
                    rstr += token;

                } else {
                    Object value = theSymbolMap.get(token);
                    if (null == value) {
                        Logger.global().capture("TemplateParserReader.translateLine: "
                                                + "unrecognized symbol " + token + " (line: '" + theLine + "')", 
                                                null,
                                                Logger.L_MAILER );
                        value = SYMBOL_NOT_FOUND;
                    }
                    rstr += value.toString();
                    if ( !st.hasMoreTokens() || !st.nextToken().equals(SYMBOL_DELIM)) {
                        Logger.global().capture("TemplateParserReader.translateLine: "
                                                + "unterminated symbol substitution: '" + theLine + "'",
                                                null,
                                                Logger.L_MAILER);
                    }
                }            
            } else {
                rstr += token;
            }
        }

        return rstr;
    }
}
